#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
    Luciole constants 
"""

# standard library imports
from gettext import gettext as _
import os.path

# related third party imports
import pygst
pygst.require('0.10')
import gst

# local application/library specific imports
# N/A


# Application name
APP_NAME = "luciole"


########################################
# Hardware constants
########################################
FAKE = 0
DVCAM = 1
WEBCAM = 2
DIGICAM = 3
HARD_TYPE = (FAKE, DVCAM, WEBCAM, DIGICAM)
HARD_TYPE_NAME = (
        "FAKE",
        "DVCAM",
        "WEBCAM",
        _("OTHER DEVICE"),
        )

########################################
# Image screen display constants
########################################
SCREEN_BOTH = 0     # when action on both screen is required
SCREEN_CAMERA = 1   # input stream display
SCREEN_PREVIEW  = 2 # preview screen 


########################################
# LUCIOLE PROJECT CONSTANTS
########################################
ACQUIRED_IMAGE_NAME = 'capture.jpeg'
# name for resized capture
ACQUIRED_IMAGE_NAME_RZ = 'capture_rz.jpeg'
RUSH_FILENAME_TPL = "rush_%05d.jpeg"
TMP_DIR = 'tmp'

(UNDEF_TV, CAPTURE, CHRONO) = range(3) 

########################################
# VIDEO FORMATS
########################################
VIDEO_PAL_RES = (720, 576)
VIDEO_PAL_FPS = 25
VIDEO_FPS_TABLE = {
                0 : ("0",0),
                1 : ("1",25),
                2 : ("2.5",10),
                3 : ("5",5),
                4 : ("12.5",2),
                5 : ("25",1)
                }  


########################################
# IMAGE FORMATS
########################################
# Thumb width displayed in treeviews, the height is calcuated in app
# according image ratio
THUMB_WIDTH         =   120      
# clor in rgb format muliply by 255 each level to go in gtk.gdk.Color format
THUMB_COLOR_RATIO = 256
THUMB_TEXT_COLOR    =   (
                            191 * THUMB_COLOR_RATIO,  
                            244  * THUMB_COLOR_RATIO,  
                            75 * THUMB_COLOR_RATIO
                        )     
THUMB_TEXT_SIZE     =   8200                # text size
THUMB_TEXT_FAMILY   =   'sans'              # font family



############################################
# PROGRAM_PATH 
###########################################
BASE_DIR = os.path.dirname(os.path.abspath(__file__))
THEMES_DIR = os.path.join(BASE_DIR, 'data/themes')
UI_DIR = os.path.join(BASE_DIR, 'ui')
TEMPLATE_DIR = os.path.join(BASE_DIR, 'data/templates')
IMAGE_DIR = os.path.join(BASE_DIR, 'data', 'images')
LOCALE_DIR = os.path.join(BASE_DIR, 'po')
SOUNDS_DIR = os.path.join(BASE_DIR, 'data', 'sounds')

###############################################
# For GLADE
###############################################
MAIN_GLADE_FILE = "luciole.glade"
MAIN_GLADE_FILE_PATH = os.path.join(UI_DIR, MAIN_GLADE_FILE)
MAIN_WINDOW_NAME = 'window1'



ACQ_MODE_GLADE_FILE = "acquisition_mode.glade"
ACQ_MODE_GLADE_PATH = os.path.join(UI_DIR, ACQ_MODE_GLADE_FILE)

NO_PROJECT_MODE_GLADE_FILE = "no_project_mode.glade"
NO_PROJECT_MODE_GLADE_PATH = os.path.join(UI_DIR, NO_PROJECT_MODE_GLADE_FILE)

VIEWER_GLADE_FILE = "viewer.ui"
VIEWER_GLADE_PATH = os.path.join(UI_DIR, VIEWER_GLADE_FILE)


MODE_MAIN_WINDOW = 'window_main'


###############################################
# GST constants
###############################################
CLOCK_TIME_NONE = gst.CLOCK_TIME_NONE

###############################################
# SOUNDS_FILE
###############################################
SOUND_SNAPSHOT_FILE = 'camera.ogg'
SOUND_SNAPSHOT_PATH = os.path.join(SOUNDS_DIR, SOUND_SNAPSHOT_FILE)

SOUND_TRACK_NAME = 'sound_track.ogg'
#################################################
# For mixer
################################################
IMAGE2MIX_NAME = 'ToMix.jpeg'
DEFAULT_IMAGE2MIX = os.path.join(IMAGE_DIR,'white.jpg')
DEFAULT_ALPHA_IMAGE = 0.5
DEFAULT_ALPHA_STREAM = 1.0

