# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
ctrl_base.py :
    Base class for ctrl class
"""
# standard library imports

# related third party imports
from pitivi.signalinterface import Signallable
from pitivi.log.loggable import Loggable
# local application/library specific imports


class ProjectMode(Signallable, Loggable) :
    """ Base class """
    __signals__ = {}
    
    def __init__(self, ctrl_project,  mode_type = None) :
        Signallable.__init__(self)
        Loggable.__init__(self)
        self.mode_type = mode_type
        self.ctrl_project = ctrl_project
        self.gui =  ctrl_project.gui
        self._viewer = None
        
        self.project = None
        self._pipeline = None

        self.connect_to_project()

    def set_viewer(self, viewer) :
        """ set the gui viewer """
        self._viewer = viewer

    def connect_to_project(self) :
        """ conecto project controller """
        self.debug('Connect to project')
        self.ctrl_project.connect('project-loaded', self.project_loaded_cb)
        self.ctrl_project.connect('project-closed', self.project_closed_cb)
    
    def project_loaded_cb(self, project_ctrl, project) :
        """ project-loaded callback """
        self.debug('RX project-loaded')
        self.prepare(project)

    def project_closed_cb(self, project_ctrl) :
        """ project-closed callback """
        self.debug('RX project-closed')
        self.release()

    def connect_to_gui(self, gui) :
        """ Meta function.
        connect to gui signals """
        self.debug('Connect to gui')
        

    def active_viewer(self) :
        """ Meta function.
            active displays associted to mode """
        pass
    
    def prepare(self, project) :
        """ Meta function.
            prepare the mode """
        
        self.debug("project : %s", project)
        self.project = project
        if project == None :
            self.error('No project to prepare')
        else :
            self.project = project


    def release(self) : 
        """ Meta function
        release """
        self.debug('Release')
        self._viewer.setPipeline(None)
        pass
        

        


