#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- mode: python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# copyright nicolas bertrand (nico@inattendu.org), 2009-2010
#
# this file is part of luciole.
#
#    luciole is free software: you can redistribute it and/or modify
#    it under the terms of the gnu general public license as published by
#    the free software foundation, either version 3 of the license, or
#    (at your option) any later version.
#
#    luciole is distributed in the hope that it will be useful,
#    but without any warranty; without even the implied warranty of
#    merchantability or fitness for a particular purpose.  see the
#    gnu general public license for more details.
#
#    you should have received a copy of the gnu general public license
#    along with luciole.  if not, see <http://www.gnu.org/licenses/>.
#
#
"""
fpi_widget.py :
	fpi ( frame per image) scale bar
"""
# standard library imports
# N/A

# related third party imports
import gobject
import gtk
from pitivi.signalinterface import Signallable
from pitivi.log.loggable import Loggable

# local application/library specific imports
import luciole.constants as LCONST

class FpiHScale(gtk.HScale, Loggable) :
    

    def __init__(self, action ) :
        Signallable.__init__(self)
        Loggable.__init__(self)
        _adj =  gtk.Adjustment(1.0, 1.0, 6.0, 1.0, 1.0, 1.0)
        gtk.HScale.__init__(self, _adj)
        self.set_size_request(70,-1)
        self.set_update_policy(gtk.UPDATE_DELAYED)
        self.connect('format-value', self._format_value_cb)
        self.connect('value-changed', self._value_changed_cb)
        action.connect_proxy(self)
        
    def _format_value_cb(self, widget, value) :
        """ 
        framerate value changed.
        This callback allow display of range [1..25 ] in the scale bar
        instead of [1..6] 
        """
        _adj = widget.get_adjustment()
        #robustness : max value of adj is 6 but range is [1..5]
        if value >= _adj.upper : 
            value = ( _adj.upper -1.00)
        if value < _adj.lower :
            value = _adj.lower
        
        (_fps_display, _fpi ) = LCONST.VIDEO_FPS_TABLE[int(value)]
        # return converted value to display  
        
        return _fps_display

    def _value_changed_cb(self, widget) :
        """framerate value changed on scrollbar """
        _value = widget.get_value()
        
        print 'value 2 :', _value
        _adj = widget.get_adjustment()
        #robustness : max value of adj is 6 but range is [1..5]yy
        if _value >= _adj.upper : 
            _value = ( _adj.upper -1.00)
        if _value < _adj.lower :
            _value = _adj.lower
        
        # force focus
        widget.set_property('has-focus', False)
        widget.set_property('has-default', False)
        
        
        # get the choosen number of frame per image value 
        (_fps_display, _fpi) = LCONST.VIDEO_FPS_TABLE[int(_value)]          

        self.emit('scale-changed', _fpi)
        # emit signal 
        #self.debug('emit change-framerate : %s', _fpi)
        #self.emit(_change_framerate, _fpi)

gobject.signal_new('scale-changed', FpiHScale,
                       gobject.SIGNAL_RUN_LAST,
                       gobject.TYPE_NONE,
                       (gobject.TYPE_PYOBJECT,))


