#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
mainwindow.py :
   GUI main window 
"""
# standard library imports
from gettext import gettext as _

# related third party imports
import gobject
import gtk
from pitivi.signalinterface import Signallable
from pitivi.log.loggable import Loggable

# local application/library specific imports
import luciole.base.exceptions as LEXCEP 
import luciole.constants as LCONST
import luciole.gui.base.builder as GUI_BUILDER
import luciole.gui.base.gui_base as GUI_BASE
import luciole.gui.control_widget as GCTRL
import luciole.gui.status_bar_widget as GUI_STATUS_BAR
import luciole.gui.treeviews.treeviews as GUI_TV
import luciole.gui.viewer_widget as GUI_VIEWER
import luciole.gui.windows.windows as GUI_WINDOWS
import luciole.gui.constants as GUI_CONST
import luciole.gui.project_modes as GUI_PJT_MODES
from luciole.gui.actions import AppActions
from luciole.gui.viewer_widget import ModeViewer



# constants
# get widget names from ui

class LucioleMainWindow(Signallable, Loggable) :
    __signals__ = {
            'open-project' : ["path"],
            'close-project' : ["save_before"],
            'new-project' : [],
            'start-acqusition' : [],
            'stop-acqusition' : [],
            'take-snapshot' : [],
            'active-mixer' : [],
            'deactive-mixer' : [],
            'aquisition-mode' : [],
            'timeline-mode' : [],
            'quit-app' : [],
            }
    
    def _get_status_bar(self) :
        """ status_bar getter"""
        return GUI_STATUS_BAR.StatusBarWidget(
                    self.builder.ui_objects[GUI_CONST.STATUS_BAR],
                    self.builder.ui_objects[GUI_CONST.STATUS_PROGESS_BAR])
    status_bar = property(  _get_status_bar,
                            None,
                            None,
                            """ Status bar widget """)
    
    def _get_drawarea(self) :
        """ status_bar getter"""
        return self.viewer
    drawarea = property(  _get_drawarea,
                           None,
                           None,
                           """ drawarea widget """)
    

    def __init__(self, instance) :
        Signallable.__init__(self)
        Loggable.__init__(self)
        # TODO : project mode to remove or change goal
        self._project_mode = None
        self.app = instance
        
        # load bulider (gtk.Builder)
        self.builder = GUI_BUILDER.UiBuilder( LCONST.MAIN_GLADE_FILE_PATH) 
        self._gui_objects = self.builder.ui_objects

        self.connect_to_app(self.app)
        # load windows manager 
        self.windows = GUI_WINDOWS.Windows(self.builder.window, 
                                                self.builder.ui_objects )
        

        #load GUI signals controllers
        self.gui_actions = AppActions(self, self.app)
        
        # load "mode" widget. allow cahne of display according project modes
        # i.e NO_PROJECT, ACQUISITION, TIMELINE
        # self._project_mode = GUI_PJT_MODES.ProjectModeWidget(
        #        self.builder.ui_objects[GUI_CONST.PROJECT_MODE_CONTAINER],
        #        self.gui_actions,
        #        self.app)
        self.viewer = ModeViewer(self.gui_actions)
        self.viewer.show_all()
        # add viewer to gui$
        _parent = self.builder.ui_objects[GUI_CONST.PROJECT_MODE_CONTAINER]
        _parent.add(self.viewer)

        # load treeviews
        self.treeviews = GUI_TV.Treeviews(self._gui_objects, self.gui_actions, self.app)
        
    def connect_to_app(self, app) : 
        app.project_ctrl.connect('project-loaded', self.project_loaded_cb)
        app.project_ctrl.connect('project-modified', self.project_modified_cb)
        app.project_ctrl.connect('project-closing', self.project_closing_cb)


        
    def project_loaded_cb(self, ctrl, project ):
        """ load project on GUI """
        # update framerate
        #self._project_mode.update_framerate(project.props['fpi'])
        # set program bar
        self.set_app_title(   project.props['project_name'], 
                                project.props['is_modified'])

    def project_modified_cb(self, project_ctrl, is_modified) :
        
        self.debug('RX signal project-modified : is_modified = %s',
                    is_modified)
        self.set_app_title(
                project_ctrl.project.props['project_name'], 
                is_modified)

    def project_closing_cb(self,  project_ctrl, project) :
        self.debug('RX signal project-closing : closing project = %s',
                    project.props['project_name'])
         
        _res = self.windows.question_message(_('Save Project before closing'))
        return _res



    def set_app_title(self, text, is_modified = False) :
        """ Set the main window  title """
        if is_modified == True :
            self.builder.window.\
                set_title( u"%s \u2014%s (*)" % (LCONST.APP_NAME, text) )
        else :
            self.builder.window.\
                set_title( u"%s \u2014 %s" % (LCONST.APP_NAME, text) )

        
   


        
