#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#

"""
viewer_widget.py 

Started on  ven 28 déc 2007 12:57:52 @USER-NAME@
Last update ven 28 déc 2007 12:57:52 @USER-NAME@

@Author Nico 
@version 1
"""
# standard library imports

from gettext import gettext as _

# related third party imports
import gobject
import gtk
from gtk import gdk
import gst
from pitivi.ui.viewer import PitiviViewer, PlayPauseButton, ViewerWidget
from pitivi.utils import time_to_string

from pitivi.log.loggable import Loggable

# local application/library specific imports
import luciole.gui.constants as GUI_CONST
import luciole.gui.fpi_widget as FPI_WDG
import luciole.constants as LCONST

class ModeViewer(PitiviViewer) :
    _NOPROJECT_WIDGETS = []
    _ACQUISITION_WIDGETS = []
    _TIMELINE_WIDGETS = []
    
    _MODE_WIDGETS = {
        'NO_PROJECT' : _NOPROJECT_WIDGETS,
        'ACQUISITION' : _ACQUISITION_WIDGETS,
        'TIMELINE' : _TIMELINE_WIDGETS,
        }


    def __init__(self, gui_actions = None , action=None, pipeline=None):
        self._mode = "NO_PROJECT" 
        self.gui_actions = gui_actions
        PitiviViewer.__init__(self, action, pipeline)
    
    def _createUi(self):
        """ Creates the Viewer GUI """
        self.debug("creating UI")
        self.builder =  gtk.Builder()
        # Add the UI objects (widgets) from the Glade XML file
        self.builder.add_from_file(LCONST.VIEWER_GLADE_PATH)
        self.builder.connect_signals(self)
        
        _main_wdg = self.builder.get_object("vbox1")
        self.pack_start(_main_wdg, expand=True)

        # drawing area
        self.aframe =  self.builder.get_object("aspectframe_viewer")
        self.drawingarea = ViewerWidget(self.action)
        self.aframe.add(self.drawingarea)
        
        
        self.configure_widgets()
        
        self._btns_frame = self.builder.get_object("alignment_modes")
        self._btns_mode = {
                        'NO_PROJECT' : self.builder.get_object("hbuttonbox_noproject"),
                        'ACQUISITION' : self.builder.get_object("alignment_acquisition"),
                        'TIMELINE' :  self.builder.get_object("alignment_timeline"),
                        }
        
        
        # by default set no project mode
        self._btns_frame.add(self._btns_mode['NO_PROJECT'])

        
        self._haveUI = True

        screen = gdk.screen_get_default()
        height = screen.get_height()
        if height >= 400:
            # show the controls and force the aspect frame to have at least the same
            # width (+110, which is a magic number to minimize dead padding).
            self._btns_frame.show_all()
            width, height = self._btns_frame.size_request()
            width += 110
            height = int(width / self.aframe.props.ratio)
            self.aframe.set_size_request(width , height)
        
        
        self.show_all()
    
    def _createUiNoProjectBox(self) :
        bbox = gtk.HBox()
        boxalign = gtk.Alignment(xalign=0.5, yalign=0.5)
        boxalign.add(bbox)
        
        self.new_button =  gtk.ToolButton(gtk.STOCK_NEW)
        self._connect_to_action(GUI_CONST.ACTION_NEW, self.new_button)
        self.new_button.set_sensitive(False)
        bbox.pack_start(self.new_button, expand=False)
        
        self.open_button =  gtk.ToolButton(gtk.STOCK_NEW)
        self._connect_to_action(GUI_CONST.ACTION_OPEN, self.open_button )
        self.open_button.set_sensitive(False)
        bbox.pack_start(self.open_button, expand=False)

        self._NOPROJECT_WIDGETS = [
            self.new_button,
            self.open_button,
            ]
        self._MODE_WIDGETS['NO_PROJECT'] = self._NOPROJECT_WIDGETS
        return boxalign

   
    def _createUiAcqBox(self) :
        bbox = gtk.HBox()
        boxalign = gtk.Alignment(xalign=0.5, yalign=0.5)
        boxalign.add(bbox)

        self.startstop_button =  gtk.ToggleButton()
        self._connect_to_action(GUI_CONST.ACTION_ACQUISITION, self.startstop_button )
        bbox.pack_start(self.startstop_button, expand=False)
        self.startstop_button.set_sensitive(False)

        self.snapshot_button =  gtk.ToolButton(gtk.STOCK_ORIENTATION_LANDSCAPE)
        self._connect_to_action(GUI_CONST.ACTION_SNAPSHOT, self.snapshot_button )
        bbox.pack_start(self.snapshot_button, expand=False)
        self.snapshot_button.set_sensitive(False)
        
        self.mixer_button =  gtk.CheckButton()
        self._connect_to_action(GUI_CONST.ACTION_MIXER, self.mixer_button )
        bbox.pack_start(self.mixer_button, expand=False)
        self.mixer_button.set_sensitive(False)
        
        self._ACQUISITION_WIDGETS = [
            self.startstop_button,
            self.snapshot_button,
            self.mixer_button
            ]

        self._MODE_WIDGETS['ACQUISITION'] = self._ACQUISITION_WIDGETS
        
        return boxalign
        
    
    def configure_widgets(self) :
        self.slider =  self.builder.get_object("slider")


        self.goToStart_button  = self.builder.get_object("toolbutton1")
        self.back_button  = self.builder.get_object("toolbutton2")
        self.forward_button  = self.builder.get_object("toolbutton4")
        self.goToEnd_button  = self.builder.get_object("toolbutton5")

        # set play pause button 
        _toolbar = self.builder.get_object("toolbar1")
        _toolbutton = self.builder.get_object("toolbutton3")
        _toolbar.remove(_toolbutton)
        self.playpause_button = PlayPauseToolButton()
        self.playpause_button.connect("play", self._playButtonCb)
        _toolbar.insert(self.playpause_button, 2)

        
        self.timelabel = self.builder.get_object("timelabel")
        self.durationlabel = self.builder.get_object("durationlabel")

        _button = self.builder.get_object("start_stop_button")
        self._connect_to_action(GUI_CONST.ACTION_ACQUISITION, _button, True )

        _button = self.builder.get_object("snapshot_button")
        self._connect_to_action(GUI_CONST.ACTION_SNAPSHOT, _button, True )
        
        self.fpi_scale = FPI_WDG.FpiHScale(self.gui_actions.get_action(GUI_CONST.ACTION_FPI))
        _box = self.builder.get_object("hbox5")
        _box.pack_start(self.fpi_scale)

        # slider
        self.posadjust= self.builder.get_object("posadjust")
        self.moving_slider = False

    def _createUiTimelineBox(self) :
        #
        # Main Vbox
        #

        vbox =  gtk.VBox()
        vboxalign = gtk.Alignment(xalign=0.5, yalign=0.5)
        vboxalign.add(vbox)

        
        # 
        # first row if vbox
        # 
        bbox = gtk.HBox()
        boxalign = gtk.Alignment(xalign=0.5, yalign=0.5)
        boxalign.add(bbox)

        self.goToStart_button = gtk.ToolButton(gtk.STOCK_MEDIA_PREVIOUS)
        self.goToStart_button.connect("clicked", self._goToStartCb)
        self.goToStart_button.set_tooltip_text(_("Go to the beginning of the timeline"))
        self.goToStart_button.set_sensitive(True)
        bbox.pack_start(self.goToStart_button, expand=False)

        self.back_button = gtk.ToolButton(gtk.STOCK_MEDIA_REWIND)
        self.back_button.connect("clicked", self._backCb)
        self.back_button.set_tooltip_text(_("Go back one second"))
        self.back_button.set_sensitive(False)
        bbox.pack_start(self.back_button, expand=False)

        self.playpause_button = PlayPauseButton()
        self.playpause_button.connect("play", self._playButtonCb)
        bbox.pack_start(self.playpause_button, expand=False)
        self.playpause_button.set_sensitive(False)

        self.forward_button = gtk.ToolButton(gtk.STOCK_MEDIA_FORWARD)
        self.forward_button.connect("clicked", self._forwardCb)
        self.forward_button.set_tooltip_text(_("Go forward one second"))
        self.forward_button.set_sensitive(False)
        bbox.pack_start(self.forward_button, expand=False)

        self.goToEnd_button = gtk.ToolButton(gtk.STOCK_MEDIA_NEXT)
        self.goToEnd_button.connect("clicked", self._goToEndCb)
        self.goToEnd_button.set_tooltip_text(_("Go to the end of the timeline"))
        self.goToEnd_button.set_sensitive(False)
        bbox.pack_start(self.goToEnd_button, expand=False)

        # current time
        self.timelabel = gtk.Label()
        self.timelabel.set_markup("<tt>00:00:00.000</tt>")
        self.timelabel.set_alignment(1.0, 0.5)
        bbox.pack_start(self.timelabel, expand=False, padding=10)
        
        sep_label = gtk.Label()
        sep_label.set_markup("/")
        sep_label.set_alignment(1.0, 0.5)
        bbox.pack_start(sep_label, expand=False, padding=5)

        self.durationlabel = gtk.Label()
        self.durationlabel.set_markup("<tt>00:00:00.000</tt>")
        self.durationlabel.set_alignment(1.0, 0.5)
        bbox.pack_start(self.durationlabel, expand=False, padding=10)
        
        vbox.pack_start(boxalign)

        #
        # second row
        #
        bbox = gtk.HBox()
        boxalign = gtk.Alignment(xalign=0.5, yalign=0.5)
        boxalign.add(bbox)
        vbox.pack_start(boxalign)
        
        self.activesound_check =  gtk.CheckButton()
        self._connect_to_action(GUI_CONST.ACTION_SOUND_ACTIVE, self.activesound_check )
        bbox.pack_start(self.activesound_check, expand=False)
        self.activesound_check.set_sensitive(False)
        
        self.stopsound_check =  gtk.CheckButton()
        self._connect_to_action(GUI_CONST.ACTION_SOUND_STOP_WITH_VIDEO, self.stopsound_check )
        bbox.pack_start(self.stopsound_check, expand=False)
        self.stopsound_check.set_sensitive(False)
        
        self.fpi_scale = FPI_WDG.FpiHScale(self.gui_actions.get_action(GUI_CONST.ACTION_FPI))
        bbox.pack_start(self.fpi_scale, expand=False)
        self.fpi_scale.set_sensitive(False)


        self._TIMELINE_WIDGETS =  [self.slider, self.goToStart_button, self.back_button,
                         self.playpause_button, self.forward_button,
                         self.goToEnd_button, self.timelabel, self.durationlabel, 
                         self.activesound_check, self.stopsound_check,  self.fpi_scale]
        self._MODE_WIDGETS['TIMELINE'] = self._TIMELINE_WIDGETS
        for _item in  self._MODE_WIDGETS['TIMELINE'] :
            if _item == self.goToStart_button :
                self.log('Found goToStart')

        return vboxalign
 
    def _connect_to_action(self, action, widget, create_icon = False ) :
        if self.gui_actions :
            _action = self.gui_actions.get_action(action)
            if _action :
                self.debug('connect button %s to action %s', widget , _action.get_name())  
                _action.connect_proxy(widget)
                #if create_icon == True :
                #    widget.get_settings().props.gtk_button_images = True
                #    print _action.create_icon(gtk.ICON_SIZE_BUTTON)
                #    widget.set_image(_action.create_icon(gtk.ICON_SIZE_BUTTON))
        else :
            self.warning('No actions defined') 


   
    
    
    def _snapshotCb(self, unused_button):
        pass

    
    def setPipeline(self, pipeline, mode="TIMELINE" ):
        """
        Set the Viewer to the given Pipeline.

        Properly switches the currently set action to that new Pipeline.

        @param pipeline: The Pipeline to switch to.
        @type pipeline: L{Pipeline}.
        """
        self.debug("self.pipeline:%r, pipeline:%r, mode %s", self.pipeline, pipeline, mode)

        if pipeline is not None and pipeline == self.pipeline:
            return
        
       
        if self.pipeline != None:
            # remove previously set Pipeline
            self._disconnectFromPipeline()
            # make ui inactive
            self._setUiActive(False)
            # finally remove previous pipeline
            self.pipeline = None
            self.currentState = gst.STATE_PAUSED
            self.playpause_button.setPause()
         
        # for image view no mode change
        if mode != 'IMAGE_VIEW' :
            self._mode = mode

        self._connectToPipeline(pipeline)
        self.pipeline = pipeline
        if self.pipeline is not None or self._mode == "NO_PROJECT":
            self._setUiActive(True)


    def _connectToPipeline(self, pipeline):
        self.debug("pipeline:%r", pipeline)
        if self.pipeline != None:
            raise ViewerError("previous pipeline wasn't disconnected")
        self.pipeline = pipeline
        if self.pipeline == None:
            return
        if self._mode == 'TIMELINE' :
            self.debug("Connect to pipeline TIMELINE : %r", pipeline)

            self._connectToPipeline_timeline()
        else :
            self.debug("Connect to pipeline ACQUISITION : %r", pipeline)
            self._connectToPipeline_acq()
            
        # if we have an action set it to that new pipeline
        if self.action:
            self.pipeline.setAction(self.action)
            self.action.activate()

    def _disconnectFromPipeline(self):
        self.debug("pipeline:%r", self.pipeline)
        if self.pipeline == None:
            # silently return, there's nothing to disconnect from
            return
        if self.action and (self.action in self.pipeline.actions):
            # if we have an action, properly remove it from pipeline
            if self.action.isActive():
                self.pipeline.stop()
                self.action.deactivate()
            self.pipeline.removeAction(self.action)
        
        if self._mode == 'TIMELINE' :
            self._disconnectFromPipeline_timeline()
        else :
            self._disconnectFromPipeline_acq()

        self.pipeline.stop()
        self.pipeline = None


    def _connectToPipeline_acq(self) :
        self.pipeline.deactivatePositionListener() 
        self.pipeline.connect('element-message', self._elementMessageCb)
    
    def _disconnectFromPipeline_acq(self) :
        self.pipeline.disconnect_by_function(self._elementMessageCb)

    def _connectToPipeline_timeline(self) :
        self.pipeline.connect('position', self._posCb)
        self.pipeline.activatePositionListener()
        self.pipeline.connect('state-changed', self._currentStateCb)
        self.pipeline.connect('element-message', self._elementMessageCb)
        self.pipeline.connect('duration-changed', self._durationChangedCb)
        self.pipeline.connect('eos', self._eosCb)
    
    def _disconnectFromPipeline_timeline(self) :
        self.pipeline.disconnect_by_function(self._posCb)
        self.pipeline.disconnect_by_function(self._currentStateCb)
        self.pipeline.disconnect_by_function(self._elementMessageCb)
        self.pipeline.disconnect_by_function(self._durationChangedCb)
        self.pipeline.disconnect_by_function(self._eosCb)




    def _setUiActive(self, active=True ) :
        self.debug("active %r, mode : %s ", active, self._mode )
        self.set_sensitive(active)
        if self._haveUI :
            if self._mode in ("TIMELINE", "ACQUISITION", "NO_PROJECT" )  :
                _child = self._btns_frame.get_child()
                if _child is not None :
                    self._btns_frame.remove(_child)
                self._btns_frame.add(self._btns_mode[self._mode])
                self._btns_frame.show_all()
                
                # active buttons
                for _item in  self._MODE_WIDGETS[self._mode] :
                    if _item == self.goToStart_button :
                        self.log('Found goToStart active =%s', active)
                    _item.set_sensitive(active)

                # deactivate others
                if active :
                    for _key in self._MODE_WIDGETS :
                        if _key != self._mode :
                            for _item in self._MODE_WIDGETS[_key]:
                                if _item == self.goToStart_button :
                                    self.log('Found goToStart deactive =%s', active)
                                _item.set_sensitive(not active)
            else :
                self.error("Unkonwn mode %s", self._mode)
        else :
            self.debug(' No UI ????')
            for item in [self.slider, self.goToStart_button, self.back_button,
                         self.playpause_button, self.forward_button,
                         self.goToEnd_button, self.timelabel, self.durationlabel,
                         self.new_button, self.open_button,
                         self.startstop_button, self.snapshot_button, self.mixer_button, 
                         self.activesound_check , self.stopsound_check ]: 
                item.set_sensitive(active)

        if active:
            self.emit("activate-playback-controls", True)


    def update_duration(self, value , frame = -1) :
        self.info("value:%s, frame:%d", gst.TIME_ARGS(value), frame)
        self.current_time = value
        self.current_frame = frame
        self.durationlabel.set_markup("<tt>%s</tt>" % time_to_string(value))
    

    def prepare_viewer(self, duration) :
        self.debug("duration : %s", gst.TIME_ARGS(duration))
        
        self.update_duration(duration)

        self.posadjust.upper = float(duration)
        

        # set first position
        self.posadjust.set_value(0)
        self._newTime(0)
        self.playpause_button.setPlay()
        
        self.debug("_initial_seek : %s", self._initial_seek )
        self.debug("Pipeline %r state %s", self.pipeline, self.pipeline.getState())

        if self._initial_seek is not None:
            seek, self._initial_seek = self._initial_seek, None
            self.debug("seek : %s", seek)
            self.pipeline.seek(seek)
    
    def _durationChangedCb(self, unused_pipeline, duration):
        self.debug("duration : %s", gst.TIME_ARGS(duration))
        position = self.posadjust.get_value()
        self.debug("position : %s", gst.TIME_ARGS(position))
        if duration < position:
            self.posadjust.set_value(float(duration))
        self.posadjust.upper = float(duration)
        self.update_duration(duration)
        if duration == 0:
            self._setUiActive(False)
        else:
            self._setUiActive(True)

        self.debug("_initial_seek : %s", self._initial_seek )
        if self._initial_seek is not None:
            seek, self._initial_seek = self._initial_seek, None
            self.debug("seek : %s", seek)
            self.pipeline.seek(seek)
      
class PlayPauseToolButton(gtk.ToolButton, Loggable):
    """ Double state gtk.Button which displays play/pause """

    __gsignals__ = {
        "play" : ( gobject.SIGNAL_RUN_LAST,
                   gobject.TYPE_NONE,
                   (gobject.TYPE_BOOLEAN, ))
        }

    def __init__(self):
        gtk.ToolButton.__init__(self)
        Loggable.__init__(self)
        #self.get_settings().props.gtk_button_images = True
        self.playing = True
        self.setPlay()
        self.connect('clicked', self._clickedCb)

    def set_sensitive(self, value):
        gtk.ToolButtonButton.set_sensitive(self, value)

    def _clickedCb(self, unused):
        self.emit("play", self.playing)

    def setPlay(self):
        """ display the play image """
        self.log("setPlay")
        if self.playing:
            self.set_stock_id(gtk.STOCK_MEDIA_PLAY)
            self.set_tooltip_text(_("Play"))
            self.playing = False

    def setPause(self):
        self.log("setPause")
        """ display the pause image """
        if not self.playing:
            self.set_stock_id(gtk.STOCK_MEDIA_PAUSE)
            self.set_tooltip_text(_("Pause"))
            self.playing = True


    






    
