#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
assistant_new_project.py :    
    GTK assistant for creation of new projects
"""
# standard library imports
import os.path
import re
from gettext import gettext as _


# related third party imports
import gtk

# local application/library specific imports
import luciole.constants as LCONST
import luciole.gui.windows.webcam_detection_widget as LWDW
import luciole.media.image as LI


class PageIntro(gtk.VBox) :
    """ Assistant introduction page """
    
    def __init__(self, assistant, project_data, *args ) :
        """ Init Introduction page of assistant"""
        
        gtk.VBox.__init__(self, homogeneous = False, spacing = 5)
        self.set_name('page_intro')
        self.assistant = assistant
        self.project_data = project_data
        
        self.pattern_filename = re.compile(r'^\w+$') 
        #
        # Initailize widgets for first page 
        #
        
        # A label 
        label = gtk.Label(
            _('This assistant will help you on configuration of a new '
                'Luciole project.'))
        self.pack_start(    child = label,
                            expand = False,
                            fill =  False,
                            padding = 10
                                )
        # A line separator
        self.pack_start(    child = gtk.HSeparator(),
                            expand = False,
                            fill =  False,
                            padding = 10
                                )
        # A label
        label = gtk.Label(_('Select project name and destination folder.'))
        self.pack_start(    child = label,
                            expand = False,
                            fill =  False,
                            padding = 10
                                )
        
        # Use a gtk.table to display project/folder chosse  
        table = gtk.Table(3, 2, True)
        
        label_project_name = gtk.Label(_('Project Name'))
        # positionning of label
        # left justification of label
        label_project_name.set_alignment(   xalign=0.0, 
                                            yalign=0.5)    
    
        # Insert label into the upper left quadrant of the table
        table.attach(label_project_name, 0, 1, 0, 1, xpadding = 10)
        
        self.entry_project_name = gtk.Entry()
        self.entry_project_name.set_text('')
        self.entry_project_name.connect('changed',  
                                        self.on_project_name_changed)
        # Insert entry_project_name into the upper right quadrant of the table
        table.attach(self.entry_project_name, 1, 2, 0, 1, xpadding =10)
        
        label_folder = gtk.Label(_('Folder'))
        # positionning of label
        # left justification of label
        label_folder.set_alignment(xalign=0.0, yalign=0.5)
        # Insert label into the lower left quadrant of the table
        table.attach(label_folder, 0, 1, 1, 2, xpadding = 10 )
        
        dialog = gtk.FileChooserDialog(
                    title = _('Select a folder'),
                    parent = None,
                    action = gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER ,
                    buttons =  (   gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,
                                    gtk.STOCK_OK, gtk.RESPONSE_OK)
                                )
        self._file_chooser = gtk.FileChooserButton(dialog)
        self._file_chooser.connect('selection-changed', self.on_dir_is_changed)

        self._file_chooser.set_current_folder(os.path.expandvars('$HOME'))
        self._file_chooser.set_filename(os.path.expandvars('$HOME'))
        # Insert label into the lower right quadrant of the table
        table.attach(   self._file_chooser,
                        1,
                        2,
                        1,
                        2,
                        yoptions=gtk.SHRINK,
                        xpadding =10)
       
        #label for message
        self.message_label =  gtk.Label('')
        # left justification of label
        self.message_label.set_alignment(xalign=0.0, yalign=0.5)
        table.attach(   self.message_label,
                        0,
                        2,
                        2,
                        3,
                        yoptions=gtk.FILL,
                        xpadding=10)


        # pack table to main widget
        self.pack_start(    child = table,
                            expand = True,
                            fill =  True,
                            padding = 10
                                )
                  
            
       
    def _is_project_exists(self, project_path) :
        """ Check if a project path exists"""
        return os.path.exists(project_path)
    
    def _is_project_name_valid(self, name) :
        """ Check ig entered project name is correct"""
        _is_valid = False
        if self.pattern_filename.match(name) :
            _is_valid = True
        return _is_valid
         

    def update_project_data(self, project_name, project_folder)  :
        """ Update project data like 'project_name' and 'project_fir'.
            projetct dict. is updated if the project project_name is correct
            and if path project_dir/project_name doe not exists yet.
            Return True if project has been updated"""

        _is_project_updated = False
        # get project name and dir
        self.message_label.set_text(' ')
        self.assistant.set_page_complete(self, False)
        # verify if input date are not balnk
        if project_name != '' and project_folder != None :
            # check projetc name validity
            if self._is_project_name_valid(project_name) :
                # check if project already exist
                fullpath = os.path.join(project_folder, project_name)
                if not self._is_project_exists(fullpath) :
                    # all cons ok update project data
                    self.project_data['project_name'] = project_name
                    self.project_data['project_dir'] = project_folder
                    self.assistant.set_page_complete(self, True)
                    _is_project_updated = True
                else :
                    # project with same name alreday exists
                    msg = _("The project %s already exists !"%(fullpath)) 
                    self.message_label.set_text(msg)
            else :
                # invalid projetc name
                msg = _("%s isn't a valid project name." 
                        "It should be a combination of alphanumeric and"
                        "'_' characters "%project_name) 
                self.message_label.set_text(msg)
        
        return _is_project_updated



            
    def on_project_name_changed(self, widget):
        """
        indicate project name change. 
        Set page complete if non blank text is set.
        """
        l_project_name =  widget.get_text()
        l_project_folder = self._file_chooser.get_filename()
        self.update_project_data(l_project_name, l_project_folder)
 
    def on_dir_is_changed(self, widget) :
        """ Call back for 'selection-changed' signal """
        l_project_folder = widget.get_filename()
        l_project_name =  self.entry_project_name.get_text()
        self.update_project_data(l_project_name, l_project_folder)

 


class PageHw(gtk.VBox) :
    """ Asistant page for hardware selection and poject image rate """
    
    def __init__(self, assistant, project_data, *args ) :
        """ Init page for hardware selection """
        gtk.VBox.__init__(self, *args )
        self.set_name('page_matos')
        self.assistant = assistant
        self.project_data = project_data
                
        #
        # set default values
        #
        
        # webcam by default 
        self.project_data['hardtype'] = LCONST.WEBCAM
        
        # choose of a value in the middle of the progress bar 
        _h_default = 3
        
        (_fps_display, _fpi) = LCONST.VIDEO_FPS_TABLE[int(_h_default)]  
        self.project_data['fpi'] = _fpi
        
        #
        # Initailize widgets for hardware selection page 
        #
        
        #
        # FIRST VBOX -- hardware selection
            
        _vbox = gtk.VBox()
        if _vbox :
            label = gtk.Label(_('Select a device'))
            _vbox.add(label)
            
            _hbox = gtk.HBox()
            if _hbox :
                _hardtype =  LCONST.DVCAM
                _main_rd_btn =  gtk.RadioButton(
                                group = None, 
                                label = LCONST.HARD_TYPE_NAME[_hardtype]
                                )
                # conect toggled signal to button
                _main_rd_btn.connect(   'toggled',
                                        self.radiobutton_toggled,
                                        _hardtype)
                _hbox.add(_main_rd_btn)
                
                _hardtype =  LCONST.WEBCAM
                _rb_btn =  gtk.RadioButton( 
                                group = _main_rd_btn,
                                label = LCONST.HARD_TYPE_NAME[_hardtype]
                                )
                # conect toggled signal to button
                _rb_btn.connect('toggled',
                                    self.radiobutton_toggled,
                                    _hardtype)
                # set webcam button as default button
                _rb_btn.set_active(True)
                _hbox.add(_rb_btn)
                
                _hardtype =  LCONST.DIGICAM
                _rb_btn =  gtk.RadioButton(
                                group=_main_rd_btn, 
                                label= " Other device.\n(Manual import)"
                                )
                # conect toggled signal to button
                _rb_btn.connect('toggled',
                                    self.radiobutton_toggled,
                                    _hardtype)
                _hbox.add(_rb_btn)

                # add 1st Hbox to main widget
                _vbox.add(_hbox)
        
            self.add(_vbox)

        #
        # SECOND VBO -- image frame rate selection   
        _vbox = gtk.VBox()
        if _vbox :
            _hbox = gtk.HBox()
            if _hbox :
                label = gtk.Label(_('Images per second'))
                _hbox.add(label)
                
                # an horizontal scale bar is used to select the framerate
                _hscale = gtk.HScale()
                #set range of fpi [1..6] to be converted to [1..25]
                # with callback scale_fps_value_changed
                _hscale.set_range(1, 6)
                _hscale.set_increments(1, 1)
                _hscale.set_value(_h_default)
                _hscale.connect('value-changed', self.scale_fps_value_changed)
                _hscale.connect('format-value', self.scale_fps_format_value)
                _hbox.add(_hscale)
                
                # add 2nd Hbox to main widget
                _vbox.add(_hbox)  
        
        self.add(_vbox)
        
        #
        # Third Vbox : Video project format
        _vbox = gtk.VBox()
        if _vbox :
            label = gtk.Label(_('Select a video project format'))
            _vbox.add(label)
            _hbox = gtk.HBox()
            if _hbox :
                _main_rd_btn =  gtk.RadioButton(
                                group = None, 
                                label = _('Input video mode')
                                )
                # conect toggled signal to button
                _main_rd_btn.connect(   'toggled',
                                        self.videoformat_radiobutton_toggled,
                                        'FULL')
                _hbox.add(_main_rd_btn)
                
                _rb_btn =  gtk.RadioButton( 
                                group = _main_rd_btn,
                                label = _('PAL format (720x576)')
                                )
                # conect toggled signal to button
                _rb_btn.connect('toggled',
                                    self.videoformat_radiobutton_toggled,
                                    'PAL')
                # set webcam button as default button
                _rb_btn.set_active(True)
                _hbox.add(_rb_btn)
                
                
                # add 1st Hbox to main widget
                _vbox.add(_hbox)
        
            self.add(_vbox)
           


    
    def radiobutton_toggled(self, widget, hw_type):
        """ Hardware radio button """
        if (widget.get_active() ) :
            self.project_data['hardtype'] = hw_type
            
    def scale_fps_value_changed(self, widget) :
        """ Fps Scale value changed"""
        value = widget.get_value()
        
        #robustness
        if value > 5 :
            value = 5
        if value < 1 : 
            value = 1 
        # get the choosen number of frame per image value 
        
        (_fps_display, _fpi) = LCONST.VIDEO_FPS_TABLE[int(value)]  
        self.project_data['fpi'] = _fpi
       
    
    def scale_fps_format_value(self, widget, value) :
        """ Fps Scale value changed.
        This callback allow display of range [1..25 ] in the scale bar instead of [1..6] 
        """
        
        #robustness
        if value > 5 : 
            value = 5
        if value < 1 : 
            value = 1  
        
        _fps_display = LCONST.VIDEO_FPS_TABLE[int(value)][0]
        # return converted value to display  
        return _fps_display

    def videoformat_radiobutton_toggled(self, widget, video_format) :
        if video_format is 'full' :
            self.project_data['resize'] = 'no'
            self.project_data['resize-mode'] = 'FULL'
        elif video_format is 'PAL' :
            self.project_data['resize'] = 'yes'
            self.project_data['resize-mode'] = 'PAL'


        
class PageWebcam(LWDW.WebcamDetectionWidget) :
    """ Assistant page for webcam detection """
    
    def __init__(self, assistant, project_data, *args ) :
        LWDW.WebcamDetectionWidget.__init__(self, project_data, *args )
        self.assistant = assistant

    def webcam_detection_prepare(self, page) :
        """ prepare assistant page for the webcam detection page """
        # during detection not indicate page complete
        self.assistant.set_page_complete(page, False)
        
        LWDW.WebcamDetectionWidget.prepare_webcam_detection(self)
   
    def _caps_selected_cb(self, webcam_props, caps) :
        LWDW.WebcamDetectionWidget._caps_selected_cb(self, webcam_props, caps)
        self.debug('project data = %s', self.project_data) 
        # almost one webcam adn hsi caps are detected
        # assisant page can now be set as complete
        self.assistant.set_page_complete(self, True)



  
class PageSummary(gtk.VBox) :
    """ Asistant summary page : show lucuile project info"""
    
    def __init__(self, assistant, project_data, *args ) :
        """ init class ofr assistnat page summary """
        gtk.VBox.__init__(self, *args)
        self.set_name('PageSummary')
        self.assistant = assistant
        self.project_data = project_data
  
        #
        # Initailize widgets for sumary page -- only a textview widget
        #
        
        self.text_wdg = gtk.TextView()
        self.add(self.text_wdg) 

    def assistant_prepare_summary_page(self) :
        """ prepare summary page """
      
        textbuffer = self.text_wdg.get_buffer()
        _str = ""
        _text_list = list()
        
        # display project name
        _str = "%s : %s" % ( _('Project Name'),
                                self.project_data['project_name']
                                )
        _text_list.append(_str)

        # display project path
        _str = "%s : %s \n" % ( _('Project Path'),
                                self.project_data['project_dir']
                                )
        _text_list.append(_str)

        # display project FPI 
        # loop on FPS TABLE to found the fps value
        _fps = "0"
        for  _value in LCONST.VIDEO_FPS_TABLE.itervalues() :
            if _value[1] == self.project_data['fpi'] : 
                _fps = _value[0]
            
        _str = "%s : %s " % ( _('Number of frames per seconds'), _fps)
        _text_list.append(_str)
        
        # display Hardware type 
        _str = "%s : %s " % \
            (   _('Hardware type'),
                LCONST.HARD_TYPE_NAME[self.project_data['hardtype']]
            )
        _text_list.append(_str)

        # display video mode 
        if self.project_data['resize'] is 'yes' :
            _str = _('Project video formatted in %s format') % \
                    self.project_data['resize-mode']
        else :
            _str = _("Project video not formatted."
                    "Input device fomat used")
            
        _text_list.append(_str)

        
        # display info specific for webcam
        if ( self.project_data['hardtype'] == LCONST.WEBCAM ) :
            # webcam name
            _str = "\n%s : %s" % (_('Webcam name'),
                                    self.project_data['webcam_data']['name'])
            _text_list.append(_str)
            
            #webcam device 
            _str = "%s : %s" % (    _('Webcam device'),
                            self.project_data['webcam_data']['device-file'])
            _text_list.append(_str)

            #webcam resolution
            
            _str = "%s : %sx%s" % (
                        _('Webcam resolution used'), 
                        self.project_data['webcam_data']['caps'][0]['width'],
                        self.project_data['webcam_data']['caps'][0]['height']
                        )
            _text_list.append(_str)

            #webcam device 
            _str = "%s : %s" % (
                    _('Webcam driver used'),
                    self.project_data['webcam_data']['v4ldriver']
                    )
            _text_list.append(_str)
        
            
            # Final message  
            _str = "\n %s" % (_('Have fun with luciole !'))
            _text_list.append(_str)

        _str = "\n".join(_text_list)         

        textbuffer.set_text(_str)

                 

class AssistantNewProject(object) :
    """ Class for managing a Gtk Assitant to configure a luciole project """
    
    __LUCIOLE_LOGO = os.path.join(LCONST.IMAGE_DIR,'luciole_logo.png')

    def __init__(self, apply_callback) :
        """ Initialize assistant window """
        self.apply_callback = apply_callback 
        
        #
        # init project datas 
        #
        self.project_data = dict()
 
        self.assistant = gtk.Assistant()
        self.assistant.connect('delete-event', self.on_quit)
        self.assistant.connect('destroy-event', self.on_quit)
        
        self.assistant.connect('apply', self.on_apply)
        self.assistant.connect('cancel', self.on_cancel)
        self.assistant.connect('close', self.on_close)
        self.assistant.connect('prepare', self.on_prepare)
        
        self.assistant.set_property('title', _('Luciole project assistant'))
        
        # assistant logo with no text
        _image = LI.Image(self.__LUCIOLE_LOGO, True, 2, False)
       

        #
        # configure page 1
        #
        self._pg1 = PageIntro( self.assistant, self.project_data)
        self.assistant.append_page(self._pg1)
        self.assistant.set_page_title( self._pg1, _(' Select a project path '))
        self.assistant.set_page_type( self._pg1,  gtk.ASSISTANT_PAGE_INTRO)
        self.assistant.set_page_side_image(self._pg1, _image.pixbuf_thumb)

        #
        # configure page 2
        #
        self._pg2 = PageHw( self.assistant, self.project_data)
        self.assistant.append_page(self._pg2)
        self.assistant.set_page_title( self._pg2, _(' Select hardware '))
        self.assistant.set_page_type( self._pg2, gtk.ASSISTANT_PAGE_CONTENT)
        self.assistant.set_page_complete( self._pg2, True)
        self.assistant.set_page_side_image(self._pg1, _image.pixbuf_thumb)
  
        #
        # configure page 3
        #
        self._pg3 = PageWebcam( self.assistant, self.project_data) 
        self.assistant.append_page(self._pg3)
        self.assistant.set_page_title( self._pg3, _(' Webcam detection '))
        self.assistant.set_page_type( self._pg3, gtk.ASSISTANT_PAGE_CONTENT)
        self.assistant.set_page_side_image(self._pg1, _image.pixbuf_thumb)
        
        #
        # configure page 4
        #
        self._pg4 = PageSummary( self.assistant, self.project_data)
        self.assistant.append_page(self._pg4)
        self.assistant.set_page_title( self._pg4, _(' Project overview '))
        self.assistant.set_page_type( self._pg4, gtk.ASSISTANT_PAGE_CONFIRM)
        self.assistant.set_page_complete( self._pg4, True)
        self.assistant.set_page_side_image(self._pg1, _image.pixbuf_thumb)
        
        
        # page_func setup : used to manage the page sequences.
        self.assistant.set_forward_page_func(self.page_func)

        self.assistant.show_all()

        

    def page_func(self, page_num) :
        """ call back to know wcih  next page to load """
        page_out = page_num
        if (page_out == 1 ) and \
           (self.project_data['hardtype'] != LCONST.WEBCAM) :
            # if web cam selected go to page 2
            # else go to page 3
            page_out = 3
        else :
            page_out = page_out +1
        return page_out


    def on_quit(self, widget, event) :
        """ Quit button clicked """
        self.assistant.destroy()

    def on_apply(self, widget) :
        """ 
        Apply button clicked.
        send project data to appliation mainframe. 
        """     
        self.apply_callback(self.project_data)       

    def on_cancel(self, widget) :
        """ Quit button clicked """
        self.assistant.destroy()

    def on_prepare(self, widget, page) :
        """ prepare callback for initiating a new page"""
        # prepare webcam detection
        if widget.get_current_page() == 2 : 
            self._pg3.webcam_detection_prepare(page)
        # prepare last page display
        if widget.get_current_page() == 3 :
            self._pg4.assistant_prepare_summary_page()
        
    def on_close(self, widget) :
        """ close action """
        self.assistant.destroy()
        
        
