#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#       :common.py
#
# Copyright (c) 2011, Nicolas Bertrand <nico@inattendu.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

"""
    common : utils/common function for interaction with gst
"""
# standard library imports
import copy 

# related third party imports
import gobject
import gst


# local application/library specific imports


def  merge_dict(elema, elemb) :
    """ merge 2 dicts """
    _copy = copy.deepcopy(elema)
    _res = merge_dict_rec(_copy, elemb)
    return _res

def merge_dict_rec(elema, elemb) :
    """ recursive function merge elema with elemb
    values in elema are the default ones. the values are updated with values in 
    elemb 
    """
    if isinstance(elemb, dict) :
        for _key in elemb.iterkeys() :
            if _key in elema :
                # key exits
                elema[_key] = merge_dict_rec(elema[_key], elemb[_key])
            else :
                elema[_key] = elemb[_key]
    else :
        elema = elemb
    
    return elema

def make_element( el_params ) :
    print el_params
    if el_params['element'] == 'capsfilter' :
        _bin = make_element_filter(el_params)
    else :
        _bin =  make_element_std(el_params)
    return _bin
def make_element_std(el_params) :
    _str = ''
    _bin = gst.element_factory_make(
            el_params['element'],
            el_params['name']
            )
    _str = "%s name=%s" % (
            el_params['element'],
            el_params['name']
            )
    if el_params.has_key('properties') :
        for _prop, _value in el_params['properties'].iteritems() :
            _bin.set_property(_prop, _value)
            _str = "%s %s=%s" % (_str, _prop, _value)
    _bin.bin_repr = _str
    return _bin

def make_element_filter(el_params)  :

    _bin = gst.element_factory_make(
            el_params['element'],
            el_params['name']
            )
    if isinstance(el_params['media_type'], gst.Caps) :
        _caps = el_params['media_type']
    else :
        _fields = []
        _fields.append(el_params['media_type'])
        if el_params.has_key('properties') :
            for _prop, _value in el_params['properties'].iteritems() :
        
                _field = " %s=%s" % (_prop, _value)
                _fields.append(_field)
    
        _caps = gst.Caps(','.join(_fields))
    _bin.set_property("caps", _caps) 

    _bin.bin_repr = _caps.to_string()
    return _bin

class BaseBin(gst.Bin) :
    """ Abstract class for in creation"""
    DEFAULT_PROPS = {}

    def __init__(self, name, params = None) :
        gst.Bin.__init__(self)
        self.params = {}
        self.bin_repr = ""
        self.set_name(name)
        self._init_params(params)
        self._makeBin()
    
    def _init_params(self, params) :
        if params is not None :
            self.params = merge_dict(
                self.DEFAULT_PROPS,
                params)
        else :
            self.debug('Using default properties')
            self.params = self.DEFAULT_PROPS
        
        self.debug("%s "%(self.params) )

    def _makeBin(self) :
        pass
    def make_ghostpads(self, sink, src) :
        """ make ghospads for bin """
        if sink is not None :
            self.add_pad(
                gst.GhostPad(
                    'sink', sink.get_pad('sink')
                    )
                )
        if src is not None :
            self.add_pad(
                gst.GhostPad(
                    'src', src.get_pad('src')
                    )
                )


   

