#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#       :webcam_bin.py
#
# Copyright (c) 2011, Nicolas Bertrand <nico@inattendu.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
"""
    Webcam input bin
"""
# standard library imports
import copy
# related third party imports
import pygst
pygst.require('0.10')

import gst
import gobject

import gtk.gdk

# local application/library specific imports
import luciole.media.lgst.common as LG_COMMON






class ImageSinkBin(LG_COMMON.BaseBin) :
    """ gst.Bin for Web cam input"""
    __gsignals__ = {
        "capture-done" : (gobject.SIGNAL_RUN_LAST,
                       gobject.TYPE_NONE,
                      (gobject.TYPE_PYOBJECT,)),
        }
  
    RATE_FILTER = {    
            'name' : 'Ratefilter',
            'element': 'capsfilter',
            'media_type' : 'video/x-raw-rgb',
            'properties' : {
                'framerate': '(fraction)1/1',
                "bpp" : 24,
                "depth": 24,
                
                },
            }
    
    IMG_SINK = {
                'name' : 'Imgsink',
                'element': 'fakesink',
                'properties' : {
                    'signal-handoffs': 'true',
                },
            }
    """
    IMG_SINK = {
                'name' : 'Imgsink',
                'element': 'filesink',
                'properties' : {
                    'location': './test.jpeg',
                },
            }
    """

    # TODO : input should be x-raw-yuv
    DEFAULT_PROPS = {
        'rate' : {
                'name' : 'ImageRate',
                'element': 'videorate',
            },
        'imgcsp' :{
                'name' : 'ImageCsp',
                'element': 'ffmpegcolorspace',
            },

        'rate_filter' : RATE_FILTER,
        
        'imgenc' : {
                'name' : 'ImgEncoder',
                'element': 'jpegenc',
                },
        'imgsink' : IMG_SINK,
            }


    
    def __init__(self, filename, params = {} ) :
        LG_COMMON.BaseBin.__init__(self, 'ImageSinkBin', params)
        self.filename = filename            
        self.capture_handler_id = 0
    
    
    def _makeBin(self) :
        self.rate = \
                LG_COMMON.make_element(self.params['rate'])
        
        self.imgcsp = \
                LG_COMMON.make_element(self.params['imgcsp'])

        self.rate_filter = \
                LG_COMMON.make_element(self.params['rate_filter'])

        #self.imgenc = \
        #        LG_COMMON.make_element(self.params['imgenc'])
        
        self.imgsink = \
                LG_COMMON.make_element(self.params['imgsink'])

 
        self.add(
            self.rate,
            self.rate_filter,
            self.imgcsp,
            #self.imgenc,
            self.imgsink, 
            )

        # make static  links
        # rate ! rate_filter  ! imgsink   
        gst.element_link_many(
            self.rate,
            self.imgcsp,
            self.rate_filter,
            #self.imgenc,
            self.imgsink, 
            )

        """       
        _str_link = ('%s ! %s ! %s' %\
                (self.rate.bin_repr,
                self.rate_filter.bin_repr,
                self.imgsink.bin_repr,
                ))

        self.debug('%s : %s'%(self.get_name(), _str_link))
        
        self.bin_repr = _str_link 
        """
        
        self.make_ghostpads(sink = self.rate, src = None)
        
        #_pad.set_blocked_async(True, self._have_data_cb)
            
    
    def _have_data_cb(self, element, data) :
        print "_have_data_cb"

    def capture(self, filename =None ) :
        if self.capture_handler_id != 0 :
            self.warning('Skipping capture request. Capture on progress')
            return
        
        #self.capture_handler_id = \
        #        self.imgenc.connect('frame-encoded', self.process_data_jpeg_cb, filename )
        
       
        
        self.capture_handler_id = \
                self.imgsink.connect('handoff', self.process_data_cb, filename )
        return

    def process_data_jpeg_cb(self, element, toto ) :
        pass
        #print element.get_pad('src')
        #print toto
        #print "frame-encoded " 


    def process_data_cb(self, gbin ,data_buffer, pad, filename ) :
        _bits_per_pixel = 8
        
        
        _caps = data_buffer.get_caps()
        print _caps.to_string()
        _structure = _caps[0]
        print _structure.to_string()
        _width = _structure["width"]
        _height = _structure["height"]
        # generate pixbuf
        self.imgsink.disconnect(self.capture_handler_id)
        self.capture_handler_id = 0
        

        # save to file 
        _file = open(self.filename,'w')
        _file.write(data_buffer)
        _file.close()

        
        
       

        
        _pixbuf = gtk.gdk.pixbuf_new_from_data(
            data_buffer.copy_on_write(),
            gtk.gdk.COLORSPACE_RGB,
            False,
            _bits_per_pixel,
            _width,
            _height,
            data_buffer.size / _height)
        
              
        # save pixbuf to file
        if filename != None :
            _filename =  filename
        else :
            _filename =  self.filename
        _pixbuf.save(self.filename, "jpeg", {"quality":"100"})
        
        self.debug('Write Jpeg on %s'%(_filename))         
        self.emit('capture-done', _filename)

        

gobject.type_register(ImageSinkBin)
gst.element_register(ImageSinkBin, 'image-sink')
    

if __name__ == '__main__':
    from luciole.media.lgst.videotest_src import VideoTestSrc
    from luciole.media.lgst.webcam_bin import WebcamBin
    
    def _timeout_cb(gbin) :
        gbin.capture()
        return True

    def _capture_done_cb(gbin, param1) :
        pass
        #print "capture done on %s" % gbin.filename

        
    def make_sink_bin(filename) :
        _bin = gst.Bin()
        _rate = gst.element_factory_make('videorate')
        
        _filter = gst.element_factory_make('capsfilter')
        _caps = gst.Caps("video/x-raw-yuv, framerate=(fraction)1/1" )
        _filter.set_property("caps", _caps)

        _sink = gst.element_factory_make('fakesink')
        #_sink.set_property('location', filename)
        
        _bin.add(
            _rate,
            _filter,
            _sink,
            )
        
        gst.element_link_many(
            _rate,
            _filter,
            _sink
            )

        _sink_pad_bin = _rate

        # sink pak 
        _bin.add_pad(
                gst.GhostPad(
                    'sink', _sink_pad_bin.get_pad('sink')
                    )
                ) 
        return _bin

    

    pipeline = gst.Pipeline()
    
    #_src = VideoTestSrc()
    _src = WebcamBin()
    _totest = ImageSinkBin('./file.jpeg')
    #_totest = make_sink_bin('./file.jpeg')
    _totest.connect('capture-done', _capture_done_cb)
    
    

    pipeline.add(_src, _totest)
    _src.link(_totest)
    
    pipeline.set_state(gst.STATE_PLAYING)
    gobject.timeout_add_seconds(2, _timeout_cb, _totest) 
    gobject.MainLoop().run()
