#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#       :webcam_bin.py
#
# Copyright (c) 2011, Nicolas Bertrand <nico@inattendu.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
"""
    Mix stream and image bin
"""
# standard library imports

# related third party imports
import pygst
pygst.require('0.10')

import gst
import gobject


# local application/library specific imports
import luciole.media.lgst.common as LG_COMMON






class ImgSrc(LG_COMMON.BaseBin) :
    """ gst.Bin for Web cam input"""

  
    FILE_SRC = {    
            'name' : 'ImgFileSrc',
            'element': 'multifilesrc',
            'properties' : {
                'location': './image.jpeg'
                },
            }
    FILE_SRC_FILTER = {    
            'name' : 'ImgFileSrcFilter',
            'element': 'capsfilter',
            'media_type' :  'image/jpeg',
            'properties' : {
                'framerate': '(fraction)2/1',
                'width' : '1280',
                'height' : '1024',
                },
            }
    IMG_CSP_FILTER = {    
            'name' : 'ImgVideoCspFilter',
            'element': 'capsfilter',
            'media_type' :  'video/x-raw-yuv',
            'properties' : {
                'format': '(fourcc)YUY2',
                },
            }

    # TODO : input should be x-raw-yuv
    DEFAULT_PROPS = {
        'filesrc' :FILE_SRC ,
        'filesrc_filter':FILE_SRC_FILTER,
        'imgdec' : {
                'name' : 'ImgDec',
                'element': 'jpegdec',
                },
        'imgcsp' : {
                'name' : 'ImgCsp',
                'element': 'ffmpegcolorspace',
                },
        'imgcsp_filter' : IMG_CSP_FILTER,
            }


    
    def __init__(self, params = {} ) :
        LG_COMMON.BaseBin.__init__(self, 'ImgSrcBin', params)
                    

    def _makeBin(self) :
        self.filesrc = \
                LG_COMMON.make_element(self.params['filesrc'])
        self.debug('creating filesrc ')

        self.filesrc_filter = \
                LG_COMMON.make_element(self.params['filesrc_filter'])
        
        self.imgdec = \
                LG_COMMON.make_element(self.params['imgdec'])
        
        self.imgcsp = \
                LG_COMMON.make_element(self.params['imgcsp'])

        self.imgcsp_filter = \
                LG_COMMON.make_element(self.params['imgcsp_filter'])

        self.add(
            self.filesrc,
            self.filesrc_filter,
            self.imgdec,
            self.imgcsp,  
            self.imgcsp_filter,
            )
        
        # make static  links
        # filesrc ! filesrc_filter ! imgdec ! imgcsp ! imgcsp_filter    
    
        self.filesrc.link_pads_full(
            "src", self.filesrc_filter, 
            "sink", gst.PAD_LINK_CHECK_NOTHING)
        
        self.filesrc_filter.link_pads_full(
            "src", self.imgdec, 
            "sink", gst.PAD_LINK_CHECK_NOTHING)
        
        self.imgdec.link_pads_full(
            "src", self.imgcsp, 
            "sink", gst.PAD_LINK_CHECK_NOTHING)
        
        self.imgcsp.link_pads_full(
            "src", self.imgcsp_filter, 
            "sink", gst.PAD_LINK_CHECK_NOTHING)
        
        _str_link = ('%s ! %s ! %s ! %s ! %s' %\
                (self.filesrc.bin_repr,
                self.filesrc_filter.bin_repr,
                self.imgdec.bin_repr,
                self.imgcsp.bin_repr,
                self.imgcsp_filter.bin_repr,
                ))
        self.debug('%s : %s'%(self.get_name(), _str_link))
        self.bin_repr = _str_link 
       
        # src pad 
        self.add_pad(
                gst.GhostPad(
                    'src', self.imgcsp_filter.get_pad('src')
                    )
                ) 



gobject.type_register(ImgSrc)
gst.element_register(ImgSrc, 'img-src-bin')
    

if __name__ == '__main__':
    import gobject
     
    pipeline = gst.Pipeline()
    _params = {} 
    #_totest = gst.element_factory_make('img-src-bin')
    _totest = ImgSrc()
    _sink = gst.element_factory_make('autovideosink') 
    
    pipeline.add(_sink, _totest)
    _totest.link(_sink)
    pipeline.set_state(gst.STATE_PLAYING)

    gobject.MainLoop().run()
