#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#       :webcam_bin.py
#
# Copyright (c) 2011, Nicolas Bertrand <nico@inattendu.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
"""
    Mix stream and image bin
"""
# standard library imports

# related third party imports
import pygst
pygst.require('0.10')

import gst
import gobject


# local application/library specific imports
import luciole.media.lgst.common as LG_COMMON
from luciole.media.lgst.image_src import ImgSrc



class MixBin(LG_COMMON.BaseBin) :
    """ gst.Bin for Web cam input"""

    VIDEOBOX = {    
            'name' : 'VideoBox',
            'element': 'videobox',
            'properties' : {
                'border-alpha': 0.0,
                'alpha': 0.5,
                },
            }
  
    DEFAULT_PROPS =  {
        'scale' :
                {
                'name' : 'MixerScale',
                'element': 'videoscale',
                },

        'videobox' : VIDEOBOX ,
        'mixer':{
                'name' : 'MixStreamAndImg',
                'element': 'videomixer',
                },
               }

    
    def __init__(self, params = {} ) :
        LG_COMMON.BaseBin.__init__(self, 'MixBin', params)
                    

    def _makeBin(self) :
        
        self.scale = \
                LG_COMMON.make_element(self.params['scale'])
        
        self.videobox = \
                LG_COMMON.make_element(self.params['videobox'])
        
        self.mixer = \
                LG_COMMON.make_element(self.params['mixer'])
        
                
        self.img_src = ImgSrc()
        
        self.add(
            self.scale, 
            self.videobox,
            self.mixer,
            self.img_src,
            )
        
        # make static  links
        # scale ! videobox !     \ 
        #                           mixer     
        # img_src ! alpha_img ! / 
        
        # TODO : The order of links imply on alpha on videobox. Why ?
        # in the other order the imageinput is displayed only
        
        gst.element_link_many(
            self.img_src,
            self.mixer
            )
        
        gst.element_link_many(
            self.scale,
            self.videobox,
            self.mixer
            )
        """
        self.img_src.link_pads_full(
            "src", self.mixer, 
            "sink", gst.PAD_LINK_CHECK_NOTHING)
        
        self.videobox.link_pads_full(
            "src", self.mixer, 
            "sink", gst.PAD_LINK_CHECK_NOTHING)
        """
        
        
        """
        self.scale.link_pads_full(
            "src", self.videobox, 
            "sink", gst.PAD_LINK_CHECK_NOTHING)

        self.videobox.link_pads_full(
            "src", self.mixer, 
            "sink", gst.PAD_LINK_CHECK_NOTHING)
        self.img_src.link_pads_full(
            "src", self.mixer, 
            "sink", gst.PAD_LINK_CHECK_NOTHING)
        """ 
        _sink_pad = self.scale
        _src_pad = self.mixer
    
        """       
        _str_link = (' %s ! %s ! %s   %s ! %s' %\
                (
                self.scale.bin_repr,
                self.videobox.bin_repr,
                self.mixer.bin_repr,
                self.img_src.bin_repr,
                self.mixer.bin_repr,
                ))

        self.debug('%s : %s'%(self.get_name(), _str_link))
        """
        # sink pad 
        self.add_pad(
                gst.GhostPad(
                    'sink', _sink_pad.get_pad('sink')
                    )
                ) 

       
        # src pad 
        self.add_pad(
                gst.GhostPad(
                    'src', _src_pad.get_pad('src')
                    )
                ) 


gobject.type_register(MixBin)

    

if __name__ == '__main__':
    import gobject
    def make_src_bin() :
        _bin = gst.Bin()
        _vtest = gst.element_factory_make('videotestsrc')
        _vtest.set_property('pattern','smpte')
        _csp = gst.element_factory_make('ffmpegcolorspace', 'inputcsp')
        _filter = gst.element_factory_make('capsfilter')
        _caps = gst.Caps(
                    "video/x-raw-yuv, width=1280 ,height=1024, "
                    "framerate=(fraction)5/1, format=(fourcc)I420"
                    )
        _filter.set_property("caps", _caps)
        _bin.add(
            _vtest,
            _csp,
            _filter)
        _vtest.link(_csp)
        _csp.link(_filter)
        # src pak 
        _bin.add_pad(
                gst.GhostPad(
                    'src', _filter.get_pad('src')
                    )
                ) 
        return _bin

    def make_sink_bin() :
        _bin = gst.Bin()
        _csp = gst.element_factory_make('ffmpegcolorspace', 'outputCsp')
        _sink = gst.element_factory_make('autovideosink')
        
        _bin.add(
            _csp,
            _sink)
        _csp.link(_sink)
        _sink_pad_bin = _csp

        # sink pak 
        _bin.add_pad(
                gst.GhostPad(
                    'sink', _sink_pad_bin.get_pad('sink')
                    )
                ) 
        return _bin


    def mix_test() :
        _bin = gst.Bin()

        _vscale =  gst.element_factory_make('videoscale')
        _videobox = gst.element_factory_make('videobox')
        _videobox.set_property('border-alpha', 0)
        _videobox.set_property('alpha', 0.9)
        
        _img_src = ImgSrc()
        #_csp = gst.element_factory_make('ffmpegcolorspace')

        #_alpha = gst.element_factory_make('alpha')
        #_alpha.set_property('alpha', 1.0)
        
        _mixer = gst.element_factory_make('videomixer')
        
        
        _bin.add(
            _vscale,
            _videobox,
            _mixer,
            #_alpha,
            _img_src
            )
        
        gst.element_link_many(
            _img_src,
            #_alpha,
            _mixer
            )

        gst.element_link_many(
            _vscale,
            _videobox,
            _mixer)
        
               
        _sink_pad = _vscale.get_pad('sink')
        _src_pad = _mixer.get_pad('src')
        
        # sink and src pad 
        _bin.add_pad(gst.GhostPad('sink', _sink_pad)) 
        _bin.add_pad(gst.GhostPad('src', _src_pad))
        return _bin


        

    pipeline = gst.Pipeline()
    
    _params = {} 
    #_src = make_src_bin()
    from luciole.media.lgst.videotest_src import VideoTestSrc
    _src = VideoTestSrc()
    _totest = MixBin(_params)
    #_totest = mix_test()
    _sink = make_sink_bin()
    
    pipeline.add(
            _src,
            _totest, 
            _sink)
    _src.link(_totest)
    _totest.link(_sink)
    pipeline.set_state(gst.STATE_PLAYING)

    gobject.MainLoop().run()
