#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#       :webcam_bin.py
#
# Copyright (c) 2011, Nicolas Bertrand <nico@inattendu.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
"""
    Webcam input bin
"""
# standard library imports

# related third party imports
import pygst
pygst.require('0.10')

import gst
import gobject


# local application/library specific imports
import luciole.media.lgst.common as LG_COMMON






class ScaleBin(LG_COMMON.BaseBin) :
    """ gst.Bin for Web cam input"""

    SCALE_BIN = {
            'name' : 'Videoscale_bin',
            'element': 'videoscale',
            'properties' : {
                'add-borders':'true',
                }
            }

    SCALE_FILTER = {    
            'name' : 'Scalefilter',
            'element': 'capsfilter',
            'media_type' : 'video/x-raw-yuv',
            'properties' : {
                'width' : '720',
                'height': '576',
                'pixel-aspect-ratio': '(fraction)1/1'
                },
            }

    # TODO : input should be x-raw-yuv
    DEFAULT_PROPS = {
        'scale' : SCALE_BIN,
        'scale_filter' : SCALE_FILTER,
            }


    
    def __init__(self, params = {} ) :
        LG_COMMON.BaseBin.__init__(self, 'ScaleBin', params)
                    

    def _makeBin(self) :
        self.scale = \
                LG_COMMON.make_element(self.params['scale'])
        
        self.scale_filter = \
                LG_COMMON.make_element(self.params['scale_filter'])

        self.add(
            self.scale,
            self.scale_filter,
            )
        
        # make static  links
        # scale ! scale_filter !   
    
        self.scale.link_pads_full(
            "src", self.scale_filter, 
            "sink", gst.PAD_LINK_CHECK_NOTHING)

        _str_link = ('%s ! %s ' %\
                (self.scale.bin_repr,
                self.scale_filter.bin_repr,
                ))
        self.debug('%s : %s'%(self.get_name(), _str_link))
        
        # sink pad 
        self.add_pad(
                gst.GhostPad(
                    'sink', self.scale.get_pad('sink')
                    )
                ) 
        # src pak 
        self.add_pad(
                gst.GhostPad(
                    'src', self.scale_filter.get_pad('src')
                    )
                ) 



gobject.type_register(ScaleBin)

    

if __name__ == '__main__':
    import gobject
    def make_src_bin() :
        _bin = gst.Bin()
        _vtest = gst.element_factory_make('videotestsrc')
        _vtest.set_property('pattern','smpte')
        _csp = gst.element_factory_make('ffmpegcolorspace')
        _filter = gst.element_factory_make('capsfilter')
        _caps = gst.Caps('video/x-raw-yuv, width=1280 ,height=1024')
        _filter.set_property("caps", _caps)
        _bin.add(
            _vtest,
            _csp,
            _filter)
        _vtest.link(_csp)
        _csp.link(_filter)
        # src pak 
        _bin.add_pad(
                gst.GhostPad(
                    'src', _filter.get_pad('src')
                    )
                ) 
        return _bin
   
    pipeline = gst.Pipeline()
    _totest = ScaleBin()
    _src = make_src_bin()
    _sink = gst.element_factory_make('autovideosink')
    
    pipeline.add(_src, _totest, _sink)
    _src.link(_totest)
    _totest.link(_sink)
    pipeline.set_state(gst.STATE_PLAYING)

    gobject.MainLoop().run()
