#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#       :webcam_bin.py
#
# Copyright (c) 2011, Nicolas Bertrand <nico@inattendu.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
"""
    Fake ( videotest) src 
"""
# standard library imports

# related third party imports
import gobject
import pygst
pygst.require('0.10')

import gst


# local application/library specific imports
import luciole.media.lgst.common as LG_COMMON






class VideoTestSrc(LG_COMMON.BaseBin) :
    """ gst.Bin for Web cam input"""

  
    VIDEO_TEST_SRC = {    
            'name' : 'VideoTestSrc',
            'element': 'videotestsrc',
            'properties' : {
                'pattern': 'smpte'
                },
            }
    VIDEO_TEST_FILTER = {    
            'name' : 'VideoTestFilter',
            'element': 'capsfilter',
            'media_type' :  'video/x-raw-yuv',
            'properties' : {
                'framerate': '(fraction)2/1',
                'width' : '1280',
                'height' : '1024',
                'format' : '(fourcc)YUY2'
                },
            }

    DEFAULT_PROPS = {
        'videotest_src' : VIDEO_TEST_SRC,
        'videotest_csp':{
                'name' : 'VideoTestCsp',
                'element': 'ffmpegcolorspace',
                },
        'videotest_filter' : VIDEO_TEST_FILTER,
            }


    
    def __init__(self, params = {} ) :
        LG_COMMON.BaseBin.__init__(self, 'VideoTestSrc', params)
                    

    def _makeBin(self) :
        self.videotest_src = \
                LG_COMMON.make_element(self.params['videotest_src'])

        self.videotest_csp = \
                LG_COMMON.make_element(self.params['videotest_csp'])
        
        self.videotest_filter = \
                LG_COMMON.make_element(self.params['videotest_filter'])
        
        
        self.add(
            self.videotest_src,
            self.videotest_csp,
            self.videotest_filter,
            )
        

        # make static  links
        # videotest_src ! videotest_csp ! videotest_filter !
        gst.element_link_many(
            self.videotest_src,
            self.videotest_csp,
            self.videotest_filter
            )

                
        _str_link = ('%s ! %s ! %s ' %\
                (self.videotest_src.bin_repr,
                self.videotest_csp.bin_repr,
                self.videotest_filter.bin_repr,
                ))
        self.debug('%s : %s'%(self.get_name(), _str_link))
        self.bin_repr = _str_link 
       
        
        self.make_ghostpads(sink = None, src = self.videotest_filter)


gobject.type_register(VideoTestSrc)
gst.element_register(VideoTestSrc, 'luciole-videotestsrc')
    

if __name__ == '__main__':
     
    pipeline = gst.Pipeline()
    _params = {} 
    #_totest = gst.element_factory_make('img-src-bin')
    _totest = VideoTestSrc()
    _sink = gst.element_factory_make('autovideosink') 
    
    pipeline.add(_sink, _totest)
    _totest.link(_sink)
    pipeline.set_state(gst.STATE_PLAYING)

    gobject.MainLoop().run()
