#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#       :webcam_bin.py
#
# Copyright (c) 2011, Nicolas Bertrand <nico@inattendu.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
"""
    Webcam input bin
"""
# standard library imports

# related third party imports
import pygst
pygst.require('0.10')

import gst
import gobject


# local application/library specific imports
import luciole.constants as LCONST
import luciole.media.lgst.common as LG_COMMON






class WebcamBin(LG_COMMON.BaseBin) :
    """ gst.Bin for Web cam input"""
    
    WCAM_DRIVER = {
            'name' : 'V4LDirver',
            'element': 'v4l2src',
            'properties' : {
                'device':'/dev/video0',
                }
            }
    WCAM_FILTER = {    
            'name' : 'Webcamfilter',
            'element': 'capsfilter',
            'media_type' : 'video/x-raw-yuv',
            'properties' : {
                'width' : '640',
                'height': '480',
                },
            }

    WCAM_CSP_CONV = {    
            'name' : 'CspConversion`',
            'element': 'capsfilter',
            'media_type' : 'video/x-raw-yuv',
            }
   

    DEFAULT_PROPS = {
        'wcam_driver' : WCAM_DRIVER,
        'wcam_filter' : WCAM_FILTER,
        'wcam_csp' :{
                'type' : 'gstelement',
                'name' : 'Webcam_csp',
                'element': 'ffmpegcolorspace',
                },
        'wcam_csp_conv' : WCAM_CSP_CONV,
            }


   
    def _get_webcam_caps(self): 
        """ webcam_caps getter """
        _pad = self.wcam_driver.get_pad("src")
        _caps = _pad.get_caps()
        self.debug("webcam caps :%s"% _caps)
        return _caps 
    
    webcam_caps = property(_get_webcam_caps, None, None,
                                " webcam capabilties ")


    def __init__(self, params = {} ) :
        LG_COMMON.BaseBin.__init__(self, 'WebcamBin', params)
                    

    def _makeBin(self) :
        _bin_str = ""
        self.wcam_driver = \
                LG_COMMON.make_element(self.params['wcam_driver'])
        
        self.wcam_filter = \
                LG_COMMON.make_element(self.params['wcam_filter'])

        self.wcam_csp = \
                LG_COMMON.make_element(self.params['wcam_csp'])
        
        self.wcam_csp_conv = \
                LG_COMMON.make_element(self.params['wcam_csp_conv'])
        self.add(
            self.wcam_driver,
            self.wcam_filter,
            self.wcam_csp,
            self.wcam_csp_conv
            )
        
        # make static  links
        # wcam_driver ! wcam_filter !  wcam_csp !   wcam_csp_conv  
    
        self.wcam_driver.link_pads_full(
            "src", self.wcam_filter, 
            "sink", gst.PAD_LINK_CHECK_NOTHING)

        
        self.wcam_filter.link_pads_full(
            "src", self.wcam_csp, 
            "sink", gst.PAD_LINK_CHECK_NOTHING)

        self.wcam_csp.link_pads_full(
            "src", self.wcam_csp_conv, 
            "sink", gst.PAD_LINK_CHECK_NOTHING)
        _str_link = ('%s ! %s ! %s ! %s' %\
                (self.wcam_driver.bin_repr,
                self.wcam_filter.bin_repr,
                self.wcam_csp.bin_repr,
                self.wcam_csp_conv.bin_repr)
                )
        self.debug(' webcam bin : %s'%_str_link)
        self.add_pad(
                gst.GhostPad(
                    'src', self.wcam_csp_conv.get_pad('src')
                    )
                ) 



gobject.type_register(WebcamBin)
gst.element_register(WebcamBin, 'webcam-bin')

if __name__ == '__main__':
    import gobject
        
    pipeline = gst.Pipeline()
    _totest = gst.element_factory_make('webcam-bin') 
    _sink = gst.element_factory_make('autovideosink')
    pipeline.add(_totest, _sink)

    _totest.link(_sink)
    pipeline.set_state(gst.STATE_PLAYING)

    gobject.MainLoop().run()
