#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#       :webcam_caps.py
#
# Copyright (c) 2011, Nicolas Bertrand <nico@inattendu.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

"""
    common : utils/common function for interaction with gst
"""
# standard library imports

# related third party imports
import gobject
import gst


# local application/library specific imports





def get_all_field_values(field, caps) :
    _values = []
    for _struct in caps :
        print _struct.to_string()
        if _struct[field] not in _values : 
            _values.append(_struct[field])
    if _values == [] :
        print ' Error'
        _values = None
    elif len(_values) == 1 :
        _values = _values.pop()
    print _values 
    return _values



def diff_structs(structs_a, structs_b) :
     
    def f_has_field(field, value, args) :
        struct , diff = args
        #print struct.to_string()
        if not struct.has_field(field) :
            diff.append(field)

        return True  #foreach continue iteration 
    
    def f_has_diff_value(field_a, value, args) :
        _structs_b , diff = args
        if _structs_b.has_field(field_a) :
            if value != _structs_b[field_a] :
                diff[field_a] = [ value, _structs_b[field_a]  ]
                
        return True  #foreach continue iteration 
    
    
    if      not isinstance(structs_a, gst.Structure) \
        or \
            not isinstance(structs_a, gst.Structure):
        print " Not correct type "
        return None, None, None
    if structs_a.get_name() != structs_b.get_name() :
        print "Cna only commpare struct with same mediatype"
        return None, None, None
   

               
    _diff_a = []
    _diff_b = []
    _diff_fields = {}
    # check for fields present in a but not in b
    structs_a.foreach(f_has_field, (structs_b,_diff_a ))
    # check for fields present in b but not in a
    structs_b.foreach(f_has_field, (structs_a,_diff_b ))

    # check for field in both stucts but with different values
    structs_a.foreach(f_has_diff_value, (structs_b,  _diff_fields))
    
    return _diff_a, _diff_b, _diff_fields





    

class WebcamCaps(object) :

    def __init__(self, caps) :
        self.caps = caps
    
    def to_string(self) :
        return self.caps.to_string()
    
    def get_size(self) :
        return self.caps.get_size()
    
    def intersect(self, caps2) :
        return self.caps.intersect(caps2)

    def get_videotypes(self, filter_caps = None) :
        _mediatypes = []
        if isinstance(filter_caps, gst.Caps) :
            _caps = self.caps.intersect(filter_caps)
        else :
            _caps = self.caps

        # loop on mediatypes
        for _struct in _caps :
            _name = _struct.get_name()
            if _name not in _mediatypes :
                _mediatypes.append(_name)
        _mediatypes.sort()

        return _mediatypes

    def get_definitions(self) :
        _definitions = []
        # loop on mediatypes
        for _struct in self.caps :
            _definition = {}
            _name = _struct.get_name()
            _definition['width'] = _struct['width']
            _definition['height'] = _struct['height']
            if _definition not in _definitions :
                _definitions.append(_definition)
        
        _definitions.sort()

        return _definitions
    
    def get_caps_from_definition(self, definition) :
        """ definition is a dict with a 'width' and 'height' field """ 
        _props = ""
        if definition['width'] != None :
            _props = "%s, width=%s"%(_props, definition['width'])
        if definition['height'] != None :
            _props = "%s, height=%s"%(_props,  definition['height'])
        _caps = gst.Caps()
        for _meditaype in self.get_videotypes() :
            _str = "%s %s"%(_meditaype,_props) 
            _caps_mt = gst.caps_from_string(_str)
            _caps.append(_caps_mt)
        _inter_caps = self.caps.intersect(_caps)
        return _inter_caps

    def get_caps_from_mediatype(self, mediatype) :
        _caps = gst.caps_from_string(mediatype)
        print "_caps : " , _caps.to_string()
        _inter_caps = self.caps.intersect(_caps)
        return _inter_caps

    def get_framerates(self, filter_caps = None) :
        _framerates = []
        if isinstance(filter_caps, gst.Caps) :
            _caps = self.caps.intersect(filter_caps)
        else :
            _caps = self.caps
        
        if _caps.get_size() == 1 :
            _normalized = _caps.normalize()
            for _struct in _normalized :
                _framerate = _struct['framerate']
                if _framerate not in _framerates :
                    _framerates.append(_framerate)
        else :
            self.warning(" Can only get framerates with only one struct")

        return _framerates
    
    def get_prop_values(self, prop = None  ,  filter_caps = None) :
        _prop_values = []
        _is_range = False
        if prop == None :
            self.error('No properties selected')
            return

        if isinstance(filter_caps, gst.Caps) :
            _caps = self.caps.intersect(filter_caps)
        else :
            _caps = self.caps
        
        if prop == 'definition' :
            return _is_range, self.get_definitions()
        if prop == 'videotype' :
            return _is_range, self.get_videotypes(_caps)
        

        _normalized = _caps.normalize()
        for _struct in _normalized :
            _val = _struct[prop]
            if   prop == 'framerate'  and isinstance(_val, gst.FractionRange) :
                # TODO : special for Fraction Range
                _prop_values = [ _val.low , _val.high ]
                _is_range = True
                break
            if _val not in _prop_values :
                _prop_values.append(_val)
        print" %s = %s"%(prop, _prop_values)
        
      
        return (_is_range, _prop_values)




    



def main():
    caps_a = ("video/x-raw-rgb,  bpp=(int)24, depth=(int)24,"
            "red_mask=(int)255, green_mask=(int)65280,"
            "blue_mask=(int)16711680, endianness=(int)4321, width=(int)320,"
            "height=(int)240, interlaced=(boolean)false, "
            "framerate=(fraction){ 30/1, 55/2, 25/1, 45/2, 20/1, 35/2, 15/1,"
            "25/2, 10/1, 15/2, 5/1 }")
    caps_b = ("video/x-raw-rgb, bpp=(int)24, depth=(int)24,"
            "red_mask=(int)16711680, green_mask=(int)65280,"
            "blue_mask=(int)255, endianness=(int)4321, width=(int)320,"
            "height=(int)240, interlaced=(boolean)false, framerate=(fraction)"
            "{30/1, 55/2, 25/1, 45/2, 20/1, 35/2, 15/1, 25/2, 10/1,"
            "15/2, 5/1 }")
    caps_a = gst.Caps(caps_a)
    caps_b = gst.Caps(caps_b)
    
    diff = diff_structs(caps_a[0] , caps_b[0] )
    print diff




if __name__ == '__main__' :
    main()

