#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#       :base.py
#
# Copyright (c) 2005-2008, Edward Hervey <bilboed@bilboed.com>
#               2008,2009 Alessandro Decina <alessandro.decina@collabora.co.uk>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

# standard library imports
import os.path
# related third party imports
import gst
import gobject

from pitivi.factories.base import SourceFactory
from pitivi.stream import  VideoStream


# local application/library specific imports
import luciole.constants as LCONST
from luciole.media.lgst.webcam_bin import WebcamBin
from luciole.media.lgst.scale_bin import ScaleBin
from luciole.media.lgst.mix_stream_img_bin import MixBin
from luciole.media.lgst.videotest_src import VideoTestSrc
from luciole.media.lgst.image_save_bin import ImageSinkBin



class WebcamSourceFactory(SourceFactory) :
    """ 
        description of pipeline for   WebcamSourceFactory
        videosrc ==> singledecodebin --> queue ==> colorspace ==> capsfilter 
        ==> direct link
        --> dynmaic link
    """
    __signals__ = {
        'image-capture-done' : [],
        }

    default_duration = gst.CLOCK_TIME_NONE
    
    _WEBCAM_PARAMS = {}

    
    def _get_alpha_value(self): 
        """ alpha_value getter """
        """
        _alpha = 0.0
        _bin = self._get_display_bin()
        
        if hasattr(_bin , "alpha_value") :
            _alpha = _bin.alpha_value
        else :
            self.warning(" No alpha_value in %s",_bin )
        return _alpha
        """
        # TODO : to implement
        return 0.0

    def _set_alpha_value(self, value):
        """ alpha_value setter """
        """
        _bin = self._get_display_bin()
        if hasattr(_bin , "alpha_value") :
            _bin.alpha_value = value
        else :
            self.warning(" No alpha_value in %s",_bin )
        _bin.alpha_value = value
        """
        # TODO : to implement
        pass
    alpha_value = property(_get_alpha_value, _set_alpha_value, None,
                                " mixer alpha  value. ")
    def _get_image2mix(self): 
        """ image2mix getter : """
        return self._image2mix

    def _set_image2mix(self, value):
        """ image2mix setter """
        if os.path.exists(value) :
            self._image2mix = value 
        else :
            self.error('No image 2 mix')

    image2mix = property(_get_image2mix, _set_image2mix, None,
                                "  image to mix . ")
    
    def _get_webcam_caps(self): 
        """ webcam_caps getter """
        _topbin = self.bins[0]
        return _topbin.acq_src.video_src.webcam_caps 
    
    webcam_caps = property(_get_webcam_caps, None , None,
                                " webcam capabilties ")
    """ 
    def _set_webcam_filter(self, wcam_filter) :
        self._WEBCAM_PARAMS['wcam_filter'] = wcam_filter
    webcam_filter = property(None, _set_webcam_filter , None,
                                " webcam filter (caps) ")
    """
    
    def _get_webcam_params(self) :
        """ getter for webcam_params """
        return  self._WEBCAM_PARAMS

    def _set_webcam_params(self, params) :
        """ setter for webcam_params """
        self._WEBCAM_PARAMS = params
    webcam_params = property(_get_webcam_params, _set_webcam_params , None,
                                " webcam params")
    def _get_with_scale(self) :
        """ getter for with_scale """
        return  self._with_scale

    def _set_with_scale(self, with_scale) :
        """ setter for with_scale """
        if isinstance(with_scale, bool ) :
            self._with_scale = with_scale
        else :
            self.error(' Invalid with scale value : %s',with_scale)

    with_scale = property(_get_with_scale, _set_with_scale , None,
                                "acqusistion with scale ?")





    
    
    def _get_display_bin(self) :
        _bin = None
        _topbin = self.bins[0]
        if hasattr(_topbin, "display_filter") :
            _bin = _topbin.display_filter
        else :
            self.warning('No display_filter bin in %s', _topbin)
        return _bin
        


    def __init__(self, name="WebcamFactory", capture_path ="./capture.jpeg", with_mixer = False, image2mix = LCONST.DEFAULT_IMAGE2MIX ):
        SourceFactory.__init__(self, None , "webcamFactory")
        self.capture_path = capture_path
        # TODO : force deactivation of mixer
        self.with_mixer = False
        self._with_scale = True

        self._image2mix = image2mix
    
    def capture(self) :
        _bin = self.bins[0]
        self.debug(" launch capture " ) 
        _bin.acq_src.img_sink.capture()
    
   

     
    def makeBin(self, stream):
        
        output_stream = \
            VideoStream(
                gst.Caps("video/x-raw-yuv;video/x-raw-rgb"), 
                pad_name="src")
        self.addOutputStream(output_stream)
        
        self.debug("output_stream:%r", output_stream)
        if output_stream is not None:
            compatible_stream = self.output_streams[0]
        bin = self._makeBin(compatible_stream)
        bin.factory = self
        self.debug("bin %s factory %s", bin, bin.factory )
        self.bins.append(bin)
        self.current_bins += 1
        self.debug('Number of bins %s, bins %s',self.current_bins, self.bins) 
        
        return bin

    def _makeAcqBin(self) :
        
        # TODO : deactivation of mixer. does not work yet

        _bin = gst.Bin()
        
       
        _bin.video_src = WebcamBin(self._WEBCAM_PARAMS)
        
        if self._with_scale == True :
            _bin.scale = ScaleBin()
        else :
            _bin.scale = gst.element_factory_make("identity","NoScaleBin")
        #_bin.video_src = VideoTestSrc()
        #_bin.mix = MixBin()
        _bin.img_sink = ImageSinkBin(self.capture_path)
        _bin.img_sink.connect('capture-done', self._image_done_cb)
        _bin.tee = gst.element_factory_make("tee","tee-acquisition")
        _bin.queue_img = gst.element_factory_make("queue","queueImageSink")
        _bin.queue_display = gst.element_factory_make("queue","queueDisplay")

        
        _bin.add(
            _bin.video_src,
            _bin.scale,
            _bin.tee,
            _bin.queue_img,
            _bin.img_sink,
            _bin.queue_display,
            #_bin.mix
            )
        
        gst.element_link_many(
                _bin.video_src,
                _bin.scale,
                _bin.tee,
                )
        
        gst.element_link_many(
                _bin.tee,
                _bin.queue_img,
                _bin.img_sink,
                )

        gst.element_link_many(
                _bin.tee,
                _bin.queue_display,
                #_bin.mix
                )
        _src_bin = _bin.queue_display

        _bin.video_src.sync_state_with_parent()
        _bin.scale.sync_state_with_parent()
        _bin.tee.sync_state_with_parent()
        _bin.queue_img.sync_state_with_parent()
        _bin.img_sink.sync_state_with_parent()
        _bin.queue_display.sync_state_with_parent()
        #_bin.mix.sync_state_with_parent()
        
        _bin.add_pad(gst.GhostPad('src', _src_bin.get_pad('src')))

        return _bin
    
    def _image_done_cb(self, bin, fname )  :
        self.debug('capture done : stored in %s',fname)
        # the execution here is not in the same thread as gui ,
        # so call capture_done with idle add
        gobject.idle_add(self._emit_image_capture_done)
    
    def _emit_image_capture_done(self) :
        self.debug("emit image-capture-done") 
        self.emit('image-capture-done')

    def _makeBin(self, output_stream) :

        video_bin = self._makeStreamBinReal(output_stream)
        
        video_bin.acq_src =self._makeAcqBin()


        video_bin.csp = gst.element_factory_make("ffmpegcolorspace",
                "internal-colorspace")
        
        video_bin.capsfilter = gst.element_factory_make("capsfilter",
                "capsfilter-proj-settings")
        self.setFilterCaps(self._filtercaps, video_bin)
        video_bin.queue = gst.element_factory_make("queue", "internal-queue")
        video_bin.queue.props.max_size_bytes = 0
        video_bin.queue.props.max_size_time = 0
        video_bin.queue.props.max_size_buffers = 3


        # add bins to topbin/video_bin 
        video_bin.add(  video_bin.decodebin,
                        video_bin.acq_src,
                        video_bin.queue,
                        video_bin.csp,
                        video_bin.capsfilter,
                        )
        
        video_bin.acq_src.link(video_bin.decodebin)

        gst.element_link_many(
            video_bin.queue,
            video_bin.csp,
            video_bin.capsfilter)

        video_bin.acq_src.sync_state_with_parent()
        video_bin.csp.sync_state_with_parent()
        video_bin.capsfilter.sync_state_with_parent()
        
        return video_bin
    
    def _makeNoMixImageBin(self) :
        _bin = gst.element_factory_make('ffmpegcolorspace','No_mix_bin')
        return _bin

   
    def _singlePadAddedCb(self, dbin, pad, topbin):
        for element in \
                [topbin.acq_src, topbin.queue, topbin.csp, topbin.capsfilter]:
            element.sync_state_with_parent()

        pad.link(topbin.queue.get_pad("sink"))
        topbin.ghostpad = gst.GhostPad("src", topbin.capsfilter.get_pad("src"))
        if pad.props.caps is not None:
            topbin.ghostpad.set_caps(pad.props.caps)
        topbin.ghostpad.set_active(True)
        topbin.add_pad(topbin.ghostpad)
    
    def _singlePadRemovedCb(self, dbin, pad, topbin):
        self.debug("dbin:%r, pad:%r, topbin:%r", dbin, pad, topbin)

        die = gst.Pad("die", gst.PAD_SRC)
        topbin.ghostpad.set_target(die)

        topbin.remove_pad(topbin.ghostpad)
        del topbin.ghostpad

        pad.unlink(topbin.queue.get_pad("sink"))

        
    def _releaseBin(self, bin):
        bin.decodebin.disconnect_by_func(self._singlePadAddedCb)
        bin.decodebin.disconnect_by_func(self._singlePadRemovedCb)


        del bin.acq_src
        del bin.decodebin
        del bin.queue
        del bin.csp
        del bin.capsfilter
        if hasattr(bin, "ghostpad"):
            # singledecodebin found something on this pad
            bin.ghostpad.set_active(False)
            bin.remove_pad(bin.ghostpad)
            del bin.ghostpad





