#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
    export_video.py :
    Manges thes export to video.
    YUV, ffmpemg, and mencoder conversion

"""
# standard library imports
import dircache
import logging 
import os
import os.path
import Queue
import re
import signal
import subprocess as SP
import threading
import time
from gettext import gettext as _

# related third party imports
import gobject

# local application/library specific imports
import luciole.base.tools as LT
import luciole.base.exceptions as L_EXCEP



(IMAGE_LIST, IMAGE_DIR) = range(2)
ERR_FILE_EXIST = 1
(EXPORT_DV, EXPORT_DVD, EXPORT_XVID) = range(3)

# Mencoder commands for export , according export type
DICT_PAL = { 
    #EXPORT_DV : "-vf scale=720:576 -ofps 25 -ovc lavc -oac pcm",
    EXPORT_DV : "-target pal-dv -aspect 4:3",
    EXPORT_DVD : 
    "-vf scale=720:576 -ofps 25 -oac pcm -ovc lavc " + \
    "-lavcopts vcodec=mpeg2video:vbitrate=9600:aspect=4/3", 
    EXPORT_XVID : "-vf scale=720:576 -ofps 25 -ovc lavc " + \
    "-lavcopts vcodec=mpeg4:vbitrate=2000:aspect=4/3 "+ \
    "-oac mp3lame -lameopts cbr:br=128", 
}

DICT_ENCODER = {
    "PAL" : DICT_PAL,
    }


class MyThreadConvertToYuv(threading.Thread):
    """ Thread class for convert a sequence of image to a YUV file """
  
    def __get_abort(self):
        """ gettter for abort """
        return self.__abort
    def __set_abort(self, value): 
        """ setter for abort """
        if value in (True, False) :
            self.__abort = value
    abort = property(
            __get_abort,
            __set_abort,
            None,
            "abort thread work")


    def __init__(self, 
                    image_input_type,
                    image_dir,
                    image_list,
                    ppmtoy4m_opts,
                    frames_per_image,
                    tmp_dir,
                    video_name,
                    finish_queue,
                    export_gui_obj):
        """ Thread class initialisation  """
        super(MyThreadConvertToYuv, self).__init__()
        self.__image_input_type = image_input_type
        self.__image_dir = image_dir
        self.__image_list = image_list
        self.__ppmtoy4m_opts = ppmtoy4m_opts
        self.__frames_per_image = frames_per_image
        self.__tmp_dir = tmp_dir
        self.__video_name = video_name
        self.__queue = finish_queue

       
        self._export_gui_obj = export_gui_obj
        self._export_gui_obj.progress_bar_text = _("Pass 1/2")
        self._export_gui_obj.progress_bar_fraction = 0

        self.__abort = False # reset abort value

    def _on_finish(self, abort):
        """Callback function. Raised when conversions is finished"""
        #preparation of next pass if no abort
        if abort == False :
            self._export_gui_obj.progress_bar_text = _("Pass 2/2") 
        self._export_gui_obj.progress_bar_fraction = 0.0
        # TBD : missing update of file path in export obj 
        self.__abort = False # reset abort value
        return False

    def _on_progress(self, value):
        """Callback function. used to know the conversion progress """
        self._export_gui_obj.progress_bar_fraction = value
        return False

    def _convert2yuv(self) :
        """ This memthod is used to convert the sequence of image in a raw
            video in YUV4MPEG format. This method use external tools as
            imagemagick and tools (ppmtoy4m) from mjpegtools suite."""
        # Get first the list of image To co_nvert is depending of the type 
    
        if self.__image_input_type == IMAGE_DIR :
            # input images are stored in a directory
            _images2conv = LT.files_in_dir(self.__image_dir, "*.jpeg")
        else :
            # input images are in a list
            _images2conv =  self.__image_list
        # copmute the number of frames 
        # to genarate = Number of images * nf of frame per image
        _nb_frames = len(_images2conv) * self.__frames_per_image
        # update the numner of frames   
        self.__ppmtoy4m_opts['-n'] = str(_nb_frames)      
    
        video_temp_base = os.path.join(self.__tmp_dir, self.__video_name)
        # build  ppmtoy4m_cmd
        ppmtoy4m_cmd = ""
        for key, value in self.__ppmtoy4m_opts.items() :
            mystring = " " + key
            if value :
                mystring = mystring + " " + value
            ppmtoy4m_cmd = mystring + ppmtoy4m_cmd
        ppmtoy4m_cmd = 'ppmtoy4m' + ppmtoy4m_cmd

        # launch ppmtoy4m, this process run during all the image convesrion
        # and receive in stdin images converted in ppm format 
        # the ppm images are sent to this subprocess by the convert operation
        # see p2 below 
        fd5 = os.open(video_temp_base + ".yuv", os.O_WRONLY|os.O_CREAT)
        fd6 = os.open(video_temp_base + ".log", os.O_WRONLY|os.O_CREAT)

        ppm_proc = SP.Popen(
                ppmtoy4m_cmd,
                shell = True,
                stdin = SP.PIPE,
                stdout = fd5,
                stderr = fd6)
        frame_cpt = 0
        for (count_image, image) in enumerate(_images2conv) :
            # check abort 
            if self.__abort == True :
                break
            #loop on images to convert
            #conversion and rezizing of cuurent image
            _img_path = os.path.join(self.__image_dir, image)
            _montage_cmd = \
                "montage  -type TrueColor -quality 100 -geometry 720x576  " +\
                _img_path + " " + video_temp_base + ".jpg"
            _sp_montage = SP.call(_montage_cmd, shell=True)
            
            _convert_cmd = "convert -type TrueColor -quality 100 " + \
                    video_temp_base + ".jpg " + video_temp_base + ".pnm "
            _sp_convert = SP.call(_convert_cmd, shell=True)
          
            for i in range(self.__frames_per_image) :
                # check abort 
                if self.__abort == True : 
                    break
                frame_cpt = frame_cpt +1
                _convert_cmd = \
                        "convert -depth 8 ppm:" + video_temp_base + ".pnm -"
                _sp = SP.Popen(
                        _convert_cmd, 
                        shell = True,
                        stdout = ppm_proc.stdin,
                        stderr = SP.PIPE)
                _sp.wait()
            # progress bar update 
            progression = ((count_image + 1.0) / len(_images2conv)) 
            gobject.idle_add(self._on_progress, progression)
        os.fsync(fd5)
        os.close(fd5)
        os.close(fd6) 
        #return the path to yuv file
        return video_temp_base + ".yuv"
  
    def run(self) :
        """ Thread start --> launch images to video conversion """
        yuvfile_path = self._convert2yuv()
        self.__queue.put(yuvfile_path)
        gobject.idle_add(self._on_finish, self.__abort)



class MyExportThread(threading.Thread):
    """ Export Thread. Thread in charge to enconde video in Yuv format
        to DV, DVD or XVID format
    """
  
    def __get_abort(self): 
        """ gettter for abort """
        return self.__abort
    def __set_abort(self, value): 
        """ setter for abort """
        if value in (True, False) : 
            self.__abort = value
    abort = property(
            __get_abort, 
            __set_abort, 
            None, 
            "abort thread work")
  
    def __init__(self, export_type, video_out_path, yuv_queue, export_gui_obj) :
        """ Init export video Thread"""
        super(MyExportThread, self).__init__()
        self.logger = logging.getLogger('luciole')
        self.__abort = False
        (self._export_type, self._video_in_path, self._video_out_path) = \
                (export_type, None, video_out_path)
        self.__queue = yuv_queue
        self._export_gui_obj = export_gui_obj
        self.__with_sound = False
        #only PAL is supported
        self.__video_format = "PAL"
        # no sound by default
        self.__with_sound = False
    
    def _on_finish(self, abort):
        """Callback function. Raised when conversions is finished"""
        # update gui progress bar
        if abort == False : 
            # Terminated normaly 
            self._export_gui_obj.progress_bar_text = _("Export Done")
            self._export_gui_obj.progress_bar_fraction = 1.0
        else :
            self._export_gui_obj.progress_bar_text = _("Export Canceled")
            self._export_gui_obj.progress_bar_fraction = 0.0
            self.__abort = False # reset abort
        return False

    def _on_progress(self, value):
        """Callback function. used to know the conversion progress """
        self._export_gui_obj.progress_bar_fraction = value
        return False

    def run(self):
        """ 
        Thread execution --> generate export 
        Build and launch ffmpeg command. 
        """
        yuv_is_finished = False
        data = None
        while (yuv_is_finished == False) :
            time.sleep(0.1)
            try :
                data = self.__queue.get(block=False)
                yuv_is_finished = True
            except Queue.Empty :
                pass
            # check abort if true leave it  
            if self.__abort == True :
                data = None
                break
        if data :
            self._video_in_path = data
            if self._export_type == EXPORT_DV :
                self._ffmpeg_launcher()
            else :
                self._mencoder_launcher()   
        gobject.idle_add(self._on_finish, self.__abort)

    def _ffmpeg_launcher(self):
        """ Video conversion with ffmpeg : only used for DV conversion """
        if ( (self._export_type == EXPORT_DV)  and \
           ( self._video_in_path )and \
           ( self._video_out_path ) ) :
            #
            # Build ffmpeg command
            #
            ffmpeg_cmd = []
            ffmpeg_cmd.append("ffmpeg")
            # add input video name
            ffmpeg_cmd.append('-i')
            ffmpeg_cmd.append(self._video_in_path)
            # add video conversion options
            coding_command = DICT_ENCODER["PAL"][EXPORT_DV]       
            for ffmpeg_w in coding_command.split() : 
                ffmpeg_cmd.append(ffmpeg_w)
            # add output video name
            ffmpeg_cmd.append(self._video_out_path)
            
            #
            # launch ffmpeg subprocess
            #
            self.logger.debug("command :%s" % ffmpeg_cmd)
            sb_encoder = SP.Popen(ffmpeg_cmd, stdout=SP.PIPE, stderr=SP.PIPE)
            
            #
            # check encoding process and finish
            #
            encode_not_finish = True
            pulse_counter = 0
            pulse_modulo = 20
            while encode_not_finish == True:
                ret_code = sb_encoder.poll()
                if type(ret_code) == int :
                    # encoding is finished when a code is returned
                    encode_not_finish = False
                if pulse_counter % pulse_modulo == 0 :
                    # update progress bar only with pulses
                    gobject.idle_add(self._on_progress, 2.0)
                pulse_counter = pulse_counter +1

    def _mencoder_launcher(self) : 
        """ 
        Video conversion with mencoder : used for convestion 
        in DVD and XVID format
        """
        if ( (self._export_type in (EXPORT_DVD, EXPORT_XVID))  and \
           ( self._video_in_path ) and ( self._video_out_path ) ) :
            #
            # Build mencoder command
            #
            mencoder_cmd = []
            mencoder_cmd.append("mencoder")
            # add export video options  
            if self.__video_format == "PAL" :
                coding_command = DICT_ENCODER["PAL"][self._export_type]       
                for mencoder_w in coding_command.split() :
                    mencoder_cmd.append(mencoder_w)
            #finally add the input and output :
            mencoder_cmd.append(self._video_in_path)
            mencoder_cmd.append('-o')
            mencoder_cmd.append(self._video_out_path)
            
            #
            # launch mencoder command
            #
            self.logger.debug("command :%s" % mencoder_cmd)
            sb_encoder = SP.Popen(mencoder_cmd, stdout=SP.PIPE, stderr=SP.PIPE)

            #
            # check encoding process and finish
            #
            encode_not_finish = True
            res_value = 0
            res_value_old = 0
            while encode_not_finish == True:
        
                # First loop until child is finished
                _buffer = []
                ret_code = sb_encoder.poll()
                if type(ret_code) == int :
                    encode_not_finish = False
                while True:
                    # second loop to detect a line
                    # remark : I have no success to use readline or readlines
                    # missing some characters
        
                    # mencoder display info on stdout, 
                    # read one char to no be blocked until eof
                    char = sb_encoder.stdout.readline(1)
                    if not char : 
                        break
                    else : 
                        if char == '\r':
                            _a_line = ''.join(_buffer) 
                            _buffer = []
                            break
                        else:
                            _buffer.append(char)
                #extact Frame num
                #print _a_line
                #Exemple line : 
    #Pos:  19.0s    475f (99%) 30.68fps Trem:   0min  65mb  A-V:0.000 [28800:0]
                # regeexp for select percent value here : 99.
                # value in parenthis ; with one or digits; select only the 
                # number 
                pattern = re.compile("\(\s*([0-9]+)\%\)")
                # get Match_object
                # cf tuto : http://docs.python.org/howto/regex.html#regex-howto
                re_res = pattern.search(_a_line)
                if re_res :
                    # result is at first index of groups method
                    # divide by 100 to have a value in range 0 .. 100
                    res_value = re_res.groups()[0]
                    if res_value != res_value_old :
                        # if send message only when percentage value change
                        # avoid sending unuseful message 
                        gobject.idle_add(   self._on_progress,
                                            float(res_value)/100.0 )
                        res_value_old = res_value
                        # check if abort requested 
                        if self.__abort == True :
                            pid = sb_encoder.pid
                            os.kill(pid, signal.SIGTERM)
                            time.sleep(1.0)
                            # check if is process really dead
                            if not isinstance(sb_encoder.poll(), int) :
                                os.kill(pid, signal.SIGKILL)
                            encode_not_finish = False
            


class ExportVideo(object) :
    """ 
    Class in charge of managing exports. The class use threads for conversions
    """

    _export_type = (EXPORT_DV, EXPORT_DVD, EXPORT_XVID) 
  
    _suffixList = (".dv", ".mpeg2", ".avi")
  
    def __get_image_dir(self):
        """ gettter for image_dir """
        return self.__image_dir
    def __set_image_dir(self, value):
        """ setter for image_dir """
        self.__image_dir = value
    def __del_image_dir(self): 
        """ deleter for image_dir """
        del self.__image_dir
    image_dir = property(    __get_image_dir, 
                            __set_image_dir,
                            __del_image_dir,
                            "Image's directory. ")

    def __get_image_list(self):
        """ gettter for image_list """
        return self.__image_list
    def __set_image_list(self, value):
        """ setter for image_list """
        self.__image_list = value
    def __del_image_list(self):
        """ deleter for image_list """
        del self.__image_list
    image_list = property(
            __get_image_list, 
            __set_image_list, 
                            __del_image_list, 
                        ("Image's list to encode. This list should be sorted."
                        "images are encoded in list order. ")
                        )

    def __get_image_input_type(self): 
        """ gettter for image_input_type """
        return self.__image_input_type
    def __set_image_input_type(self, value): 
        """ setter for image_input_type """
        if value in (IMAGE_LIST, IMAGE_DIR) :
            self.__image_input_type = value
        else :
            msg = "[", self.__class__.__name__, \
                    "] ", value, " not in correct type"
            self.logger.info(msg)
    def __del_image_input_type(self): 
        """ deleter for image_input_type """
        del self.__image_input_type
    image_input_type = property(
            __get_image_input_type, 
            __set_image_input_type,
            __del_image_input_type,
            "Image's input type  ")

    def __get_video_name(self):
        """ gettter for video_name """
        return self.__video_name
    def __set_video_name(self, value):
        """ setter for video_name """
        self.__video_name = value
    def __del_video_name(self):
        """ deleter for video_name """
        del self.__video_name
    video_name = property(
            __get_video_name, 
            __set_video_name, 
            __del_video_name, 
            "video name ")

    def __get_video_dir(self):
        """ gettter for video_dir """
        return self.__video_dir
    def __set_video_dir(self, value):
        """ setter for video_dir """
        self.__video_dir = value
    def __del_video_dir(self):
        """ deleter for video_dir """
        del self.__video_dir
    video_dir = property(
            __get_video_dir,
            __set_video_dir,
            __del_video_dir, 
            "Video directory. ")

    def __get_frames_per_image(self):
        """ gettter for frames_per_image """
        return self.__frames_per_image
    def __set_frames_per_image(self, value):
        """ setter for frames_per_image """
        self.__frames_per_image = value
    def __del_frames_per_image(self): 
        """ deleter for frames_per_image """
        del self.__frames_per_image
    frames_per_image = property(
            __get_frames_per_image,
            __set_frames_per_image,
            __del_frames_per_image,
            "Number of frames per image. ")

    def __get_output_fps(self):
        """ gettter for output_fps """
        return self.__output_fps
    def __set_output_fps(self, value):
        """ setter for output_fps """
        self.__output_fps = value
    def __del_output_fps(self): 
        """ deleter for output_fps """
        del self.__output_fps
    output_fps = property(
            __get_output_fps, 
            __set_output_fps, 
            __del_output_fps, 
            "Video out framerate ")

    def __get_video_format(self):
        """ gettter for video_format """
        return self.__video_format
    def __set_video_format(self, value): 
        """ setter for video_format """
        self.__video_format = value
        #set values for PAL format
        if self.__video_format == "PAL" :
            # framerate - see man ppmtoy4m
            self.__ppmtoy4m_opts['F'] = "-F 25:1"
            # aspect ratio - see man ppmtoy4m 
            self.__ppmtoy4m_opts['A'] = "-A 59:54"
    def __del_video_format(self):
        """ deleter for video_format """
        del self.__video_format
    video_format = property(
            __get_video_format,
            __set_video_format,
            __del_video_format, 
            "Video format ")

    def __get_video_type(self):
        """ gettter for video_type """
        return self.__video_type
    def __set_video_type(self, value): 
        """ setter for video_type """
        self.__video_type = value
    def __del_video_type(self):
        """ deleter for video_type """
        del self.__video_type
    video_type = property(
            __get_video_type, 
            __set_video_type, 
            __del_video_type, 
            "Video type ")

    def __get_video_aspect(self): 
        """ gettter for video_aspect """
        return self.__video_aspect
    def __set_video_aspect(self, value): 
        """ setter for video_aspect """
        self.__video_aspect = value
    def __del_video_aspect(self): 
        """ deleter for video_aspect """
        del self.__video_aspect
    video_aspect = property(
            __get_video_aspect, 
            __set_video_aspect, 
            __del_video_aspect, 
            "Video aspect. ")

    def __get_with_sound(self): 
        """ gettter for with_sound """
        return self.__with_sound
    def __set_with_sound(self, value): 
        """ setter for with_sound """
        self.__with_sound = value
    def __del_with_sound(self): 
        """ deleter for with_sound """
        del self.__with_sound
    with_sound = property(
            __get_with_sound, 
            __set_with_sound, 
            __del_with_sound, 
            "Video export with sound ? ")
  
    def __get_export_on_progress(self):
        """ 
        gettter for export_on_progress 
        export status is knowed by the status of the last export thread
        """
        if self._t_encoder  and self._t_encoder.isAlive() == True :
            self._export_on_progress = True
        else :
            self._export_on_progress = False
        return self._export_on_progress
    export_on_progress = property(
            __get_export_on_progress,
            None, 
            None, 
            "export status")



    def __init__(self, tmp_dir, export_gui_obj):
        """Init of class"""
        self.__tmp_dir = os.path.join(tmp_dir,"export") 
        self.logger = logging.getLogger('luciole')
        self._clean_tmp_dir()
        
        # input image list
        self.__image_list = list()
        # image type input is image_list by default
        self.__image_input_type = IMAGE_LIST

        self.__video_name = "export"
        self.__video_dir = ""
        self.__output_fps = "25"
        self.__video_format = "PAL"
        self.__video_type = "DV"
        self.__video_aspect = "4/3"

        # set ppmtoy4m options
        self.__ppmtoy4m_opts = dict()
        # framerate - see man ppmtoy4m
        self.__ppmtoy4m_opts['-F'] = "25:1"
        # aspectRatio - see man ppmtoy4m
        self.__ppmtoy4m_opts['-A'] = "59:54"
        # subsamplimg chroma mode  - see man ppmtoy4m
        self.__ppmtoy4m_opts['-S'] = "420mpeg2"
       # interlacing mode,progressive, non-interlaced  - see man ppmtoy4m
        self.__ppmtoy4m_opts['-I'] = "p"
        # total output frames  - see man ppmtoy4m
        self.__ppmtoy4m_opts['-n'] = "0"
        # verbosity  - see man ppmtoy4m
        self.__ppmtoy4m_opts['-v'] = "2"
        # default value for frame Per image
        self.__frames_per_image = 10
        # no sound by default
        self.__with_sound = False
    
        self._export_on_progress = False   # export progression
        self._t_yuv = None
        self._t_encoder = None
        self._video_path = None
        self.__image_dir = None
        if self.__video_format == "PAL":
            self.__video_res = (720, 576)
        # export gui obj    
        self._export_gui_obj = export_gui_obj

        self._data_queue = None

  
    def export(self, export_data, force_export=False) :
        """ Export function 
            exportData is a dict with the following items :
            'image_input' = type of image put list or dir (IMAGE_LIST,IMAGE_DIR)
            'image_list' = List of image to convert, each element is an absolute
                            path yo the image
                or
            'image_dir' = dir with all the images to impot : Not implemented
            'export_dir' = the directory where the video will be exported
            'video_name' = the export video name wihout extension
            'export_type' = the type of export 
                            (EXPORT_DV,EXPORT_DVD,EXPORT_XVID)
            'fpi' = the frame rate
            """

        
        (_video_exists, _video_path) = self._is_video_exists(export_data)
        if (    ( _video_exists == False) or (force_export == True) 
                and 
                ( _video_path != None ) ) : 
            # check tmp dir is clean
            self._clean_tmp_dir()
            self._export_on_progress = True

            # initiate queue for comunication between yuv converter 
            # and export converter
            self._data_queue = Queue.Queue()

            #
            # Check input parameters
            #
            self.logger.debug("export Data : %s "%export_data)
            export_is_valid = False
            if export_data.has_key('image_input') :
                self.__image_input_type = export_data['image_input']
                export_is_valid = True

            # check type of input validity
            if( 
                    (export_is_valid == True ) 
                and 
                    ( 
                    (
                    export_data.has_key('image_list') 
                    and
                    export_data['image_input'] == IMAGE_LIST
                    )
                or 
                    (
                    export_data.has_key('image_dir') 
                    and
                    ( export_data['image_input'] == IMAGE_DIR )
            ) )) :
                
                if export_data.has_key('image_list') :
                    self.__image_list = export_data['image_list']
                if export_data.has_key('image_dir') : 
                    self.__image_list = export_data['image_dir']
                export_is_valid = True
            else :
                export_is_valid = False

            if  (export_is_valid == True) and (export_data.has_key('fpi')) :
                self.__frames_per_image = int(export_data['fpi'])
                export_is_valid = True
            else :
                export_is_valid = False

            if  (export_is_valid == True) and \
                    (export_data.has_key('video_name')) :
                self.__video_name = export_data['video_name']
                export_is_valid = True
            else :
                export_is_valid = False

            if (export_is_valid == True) and os.path.exists(_video_path) :
                try :
                    os.remove(_video_path) 
                except OSError, _err :
                    _video_path = None  
                    export_is_valid = False 
                    _serr =  _('Unable to erase : %s') % _err.filename
                    _serr = "%s \n %s" % (_serr,  _err.strerror)
                    raise L_EXCEP.LucioException, _serr


            if export_is_valid == True :
                # start yuv converter sub process
                self._t_yuv = MyThreadConvertToYuv(
                        self.__image_input_type,
                        self.__image_dir,
                        self.__image_list,
                        self.__ppmtoy4m_opts,
                        self.__frames_per_image,
                        self.__tmp_dir,
                        self.__video_name,
                        self._data_queue,
                        self._export_gui_obj)
                self._t_yuv.start()
                # start video export encoder subprocess
                self._video_path = _video_path
                self._t_encoder = MyExportThread(
                        export_data['export_type'],
                        self._video_path,
                        self._data_queue,
                        self._export_gui_obj)
                self._t_encoder.start()
            else :
                return (0, _video_path)
    
        if ( _video_exists == True ) :
            return (ERR_FILE_EXIST, _video_path)
        else :
            return (0, _video_path)

    def cancel_export(self):
        """ Cancel export """
        if self._export_on_progress == True :
            # kill the eport Threads
            if self._t_yuv.isAlive() :
                self._t_yuv.abort = True 
            if self._t_encoder.isAlive() :
                self._t_encoder.abort = True 
            #check if process remains 
            while ( self._t_yuv.isAlive() or self._t_encoder.isAlive() ) : 
                time.sleep(0.1)
            # clean temporary files  
            self._clean_tmp_dir()
            try :
                os.remove(self._video_path ) 
                self.logger.debug("%s is removed" % self._video_path)
            except OSError, _err :
                _serr = _('Unable to erase : %s') % _err.filename
                _serr = "%s \n %s" % (_serr,  _err.strerror)
                raise L_EXCEP.LucioException, _serr



    ###########################################################################
    # private methods
    ########################################################################### 
    def _clean_tmp_dir(self) :
        """ Clean or create  tmp dir if needed"""
        if not os.path.exists(self.__tmp_dir) :
            #create it
            try :
                os.makedirs(self.__tmp_dir)
            except OSError, _err :
                _serr = _('Creation of export folder not possible -- %s' % \
                        _err.strerror)
                raise L_EXCEP.LucioException, _serr
        else :
            # directory exist, clean it
            _list = dircache.listdir( self.__tmp_dir)
            for _file in _list :
                _file_path = os.path.join(self.__tmp_dir, _file)
                try :
                    os.remove(_file_path) 
                    self.logger.debug("%s is removed" % _file_path)
                except OSError, _err :
                    _serr = _('Unable to erase : %s') % _err.filename
                    _serr = "%s \n %s" % (_serr,  _err.strerror)
                    raise L_EXCEP.LucioException, _serr

    def _is_video_exists(self, export_data):
        """ test if a video path exists """
        _exists = False
        _video_path = None
        # check export type 
        if export_data.has_key('export_type') and \
           export_data['export_type'] in self._export_type :
            _suffix = self._suffixList[export_data['export_type']]
            # check export video name
            if export_data.has_key('video_name') :
                video_name = export_data['video_name'] + _suffix
                
                # check export dir 
                if not os.path.exists(export_data['export_dir']) :
                    #create it
                    try :
                        LT.mkdirs(export_data['export_dir'])
                    except L_EXCEP.LucioException , _err :
                        # to robustify : Error handling
                        raise L_EXCEP.LucioException, _err
                _video_path = os.path.join(
                        export_data['export_dir'],
                        video_name)
                
                # check if video path exists
                if os.path.exists(_video_path) : 
                    _exists = True
        return (_exists, _video_path)



