#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
test_assistant_new_project.py :    
    GTK assistant for creation of new projects
"""
# standard library imports
import os.path
import shutil

# related third party imports
import pygtk
pygtk.require('2.0')
import gtk
import gobject

from pitivi.log.loggable import Loggable

# local application/library specific imports
from common import init_log, rmdir

import luciole.base.exceptions as LEXCEP
from luciole.base.tools import mkdirs, copyf
#tested module
from luciole.media.image import Image , RushImages

IMAGE_NAME = 'image_0000.jpeg'
IMAGE_PATH = os.path.join('./test/image/',IMAGE_NAME)

RUSH_DIR = './test/image/rush'
class Tester(Loggable) :

    def __init__(self) :
        Loggable.__init__(self)
        self.debug('starting tester')
        window = gtk.Window()
        window.connect('destroy', lambda w: gtk.main_quit())
        window.show_all()
        self.window = window
        gobject.timeout_add(500, self._test_1)

    def _test_1(self) :
        # Test 1
        # genereate image pixbuf with text
        _img = Image(
                IMAGE_PATH,
                generate_pixbuf = True,
                generate_text = True )
        self.debug("image name = %s",_img.name)
        _wdg = gtk.image_new_from_pixbuf(_img.pixbuf_thumb)
        self.window.add(_wdg)
        _wdg.show()
        
        gobject.timeout_add(100, self._test_2)
        return False
    
    def _test_2(self) :
        # rm rushdir
        rmdir(RUSH_DIR)
        # create rush dir
        mkdirs(RUSH_DIR)
        _rush = RushImages(RUSH_DIR)
        # copy image to rush dir
        copyf(IMAGE_PATH,
                os.path.join(RUSH_DIR,IMAGE_NAME)) 
        
        _rush.append(IMAGE_NAME)
        self.debug("rush images : %s",_rush.dump_image_name())
        return False

        


if __name__ == '__main__' :
    init_log()
    Tester()
    gtk.main()


