#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
test_assistant_new_project.py :    
    GTK assistant for creation of new projects
"""
# standard library imports
import os.path
import shutil

import glob


# related third party imports
import pygtk
pygtk.require('2.0')
import gtk
import gobject

from pitivi.log.loggable import Loggable

# local application/library specific imports
from common import init_log, rmdir
from luciole.base.tools import mkdirs

#tested module
from luciole.gui.status_bar_widget import StatusBarWidget
from luciole.media.image import RushImages
from luciole.ctrl.import_image import ImportController


IMAGES =glob.glob('./test/image/*.jpeg')


PROJECT_PROPS = {
        'project_dir' : './test/project_dir',
        'rush_dir' : 'rush',
        'resize' : 'no',
        }




class Tester(Loggable) :

    def __init__(self) :
        Loggable.__init__(self)
        self.debug('starting tester')
         # initialize threads
        gobject.threads_init() 



        window = gtk.Window()
        window.connect('destroy', lambda w: gtk.main_quit())
        
        _box = gtk.VBox()
        
        _hbox = gtk.HBox()
        status_bar = gtk.Statusbar() 
        _hbox.pack_start(status_bar)
        pbar = gtk.ProgressBar()
        _hbox.pack_start(pbar)
        
        self.status_bar = StatusBarWidget(status_bar,pbar) 
        

        self._image = gtk.Image()
        _box.pack_start(self._image)
        
        _box.pack_start(_hbox)
        
        window.add( _box)             
        window.show_all()

        
        self.rush = self.prepare_test()
        
        self.debug(' Images to import %s', IMAGES)
        importer =  ImportController(   IMAGES,
                                        PROJECT_PROPS,
                                        self.rush,
                                        self.status_bar
                                        )
        importer.connect('import-image-done', self._done_import_cb)


    def prepare_test(self) :
        _rush_dir =  os.path.join(  PROJECT_PROPS['project_dir'], 
                                    PROJECT_PROPS['rush_dir']
                                    )
        _tmp_dir =  os.path.join(  PROJECT_PROPS['project_dir'], 'tmp')

        IMAGES.sort() 
        rmdir(PROJECT_PROPS['project_dir'])
        rmdir(_tmp_dir)

        mkdirs( _rush_dir)
        mkdirs( _tmp_dir)
        
        _rush = RushImages(_rush_dir)
        return _rush
    
    def _done_import_cb(self, imager, data) :
        self.debug("import done : %s", data)
        self.debug("images in rush %s",  self.rush.dump_image_name)
        self._image.set_from_pixbuf(data[0].pixbuf_thumb)
  
if __name__ == '__main__' :
    init_log()
    Tester()
    gtk.main()


