#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
test_assistant_new_project.py :    
    GTK assistant for creation of new projects
"""
# standard library imports
import os.path
import shutil

# related third party imports
import pygtk
pygtk.require('2.0')
import gtk


from pitivi.log.loggable import Loggable

# local application/library specific imports
from common import init_log

import luciole.base.exceptions as LEXCEP

#tested module
from luciole.project.project_etree import ProjectToFile 

# imort after because pygst.require() called previously
# TODO : check in code when to correctly call  pygst.require()
import gst

PROJECT_NAME = 'test_luciole'
PROJECT_PATH = './test/'


PROJECT_DATA_NO_ACQ = {
    'project_dir': PROJECT_PATH, 
    'webcam_data': {}, 
    'project_name': PROJECT_NAME, 
    'fpi': 5,
    'resize': 'yes',
    'resize-mode': 'PAL',
    'hardtype': 3
    }

CAPS_STR = ("video/x-raw-yuv, format=(fourcc)I420, width=(int)1280,"
            "height=(int)1024, interlaced=(boolean)false,"
            "framerate=(fraction)30/1")
CAPS = gst.Caps(CAPS_STR)

PROJECT_DATA_WEBCAM = { 
    'project_dir': PROJECT_PATH,  
    'webcam_data': {
        'device-file': '/dev/video1', 
        'caps': CAPS, 
        'v4ldriver': 'v4l2src', 
        'name': 'Built-in iSight'
            }, 
    'project_name': 'test_luciole',
    'fpi': 5,
    'hardtype': 2
            }

class Tester(Loggable) :

    def __init__(self) :
        Loggable.__init__(self)
        self.debug('starting tester')
        self.info('TEST 1 : tewt with webcam data')
        self.test_etree(PROJECT_DATA_WEBCAM)

        self.info('TEST 2 : tewt with no acquisition device')
        self.test_etree(PROJECT_DATA_NO_ACQ)
        
        self.info(' TESTS finished')

    def test_etree(self, input_data) :
        _parser = ProjectToFile()
        # Test 1 : project creation
        self.del_project()
        try :
            _path = _parser.create(input_data)
        except LEXCEP.LucioException, _err :
            self.error(" Imposible to create project", _err)
            #raise LEXCEP.LucioException, _err
        else :
            self.debug('Project %s created ', _path)

        # Test 2 : open created project
        _parser.open(_path)




    def del_project(self) :
        _p_path = os.path.join(PROJECT_PATH, PROJECT_NAME)
        if os.path.exists(_p_path) :
            # delete folder
            self.debug('Deleting folder %s', _p_path)
            shutil.rmtree(_p_path)



if __name__ == '__main__' :
    init_log()
    Tester()
    #gtk.main()


