#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
test_scale_bin.py :    
    Test scale bin
"""
# standard library imports

# related third party imports
import pygst
pygst.require('0.10')

import gst
import gobject

from pitivi.log.loggable import Loggable

# local application/library specific imports
from common import init_log, rmdir

#tested module
from luciole.media.lgst.scale_bin import ScaleBin


class Tester(Loggable) :

    def __init__(self) :
        Loggable.__init__(self)
        self.debug('starting tester')
        self.pipeline = gst.Pipeline()
        _totest = ScaleBin()
        _src = self.make_src_bin()
        _sink = gst.element_factory_make('autovideosink')
    
        self.pipeline.add(_src, _totest, _sink)
        _src.link(_totest)
        _totest.link(_sink)



    def make_src_bin(self) :
        _bin = gst.Bin()
        _vtest = gst.element_factory_make('videotestsrc')
        _vtest.set_property('pattern','smpte')
        _csp = gst.element_factory_make('ffmpegcolorspace')
        _filter = gst.element_factory_make('capsfilter')
        _caps = gst.Caps('video/x-raw-yuv, width=1280 ,height=1024')
        _filter.set_property("caps", _caps)
        _bin.add(
            _vtest,
            _csp,
            _filter)
        _vtest.link(_csp)
        _csp.link(_filter)
        # src pak 
        _bin.add_pad(
                gst.GhostPad(
                    'src', _filter.get_pad('src')
                    )
                ) 
        return _bin

    def run(self) :        
        self.pipeline.set_state(gst.STATE_PLAYING)
        gobject.MainLoop().run()

if __name__ == '__main__' :
    init_log()
    tester = Tester()
    tester.run()


