#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
test_webcam_viewer.py :    
    webcam viewer tester
"""
# standard library imports
import os.path
import shutil

# related third party imports
import gobject
import pygtk
pygtk.require('2.0')
import gtk

import pygst
pygst.require('0.10')
import gst


from pitivi.log.loggable import Loggable
from pitivi.action import ViewAction
from pitivi.pipeline import Pipeline
from pitivi.ui.viewer import PitiviViewer

# local application/library specific imports
from common import init_log

import luciole.base.exceptions as LEXCEP

#tested module
from luciole.gui.webcam_properties_widget import WebcamPropsWdg      
from luciole.media.webcam_detect.webcam_detection import \
        WebcamDetection, WebcamProps

from luciole.media.lgst.webcam_factory import WebcamSourceFactory
        

class Tester(Loggable) :

    def __init__(self) :
        Loggable.__init__(self)
        self.debug('starting tester')
        # initialize threads
        gobject.threads_init() 
        self.wcam_params = {}
        self.with_scale = False
        self.mainloop = gobject.MainLoop()
    
    def run(self) :
        # generate UI
        self._createUi()
        self.webcam_props = WebcamProps(self.webcam_props_widget)

        self.webcam_props.connect('caps-selected', self._caps_selected_cb)
        # start webcam detection 
        self._wcam_detect =  WebcamDetection()
        _nb = self._wcam_detect.detect_webcams()
        if _nb > 0 :
            _webcams = [ _device['name'] \
                    for _device in self._wcam_detect.webcam_devices]
            self.webcam_props_widget.display_webcams(_webcams)
            
            self.webcam_props_widget.connect('webcam-changed', 
                    self._webcam_changed_cb)  

        # run the mainloop
        self.mainloop.run()

    def prepare_viewer(self) :
        viewer = self.ptv_viewer
        if viewer.pipeline is not None :
            try :
                viewer.pause()
            except :
                pass
        self.factory = WebcamSourceFactory()
        self.factory.webcam_params =  self.wcam_params
        self.factory.with_scale = self.with_scale

        pipeline = Pipeline()
        action = ViewAction()
        action.addProducers(self.factory)
        viewer.setPipeline(None)
        viewer.showSlider()
        
        # FIXME: why do I have to call viewer.setAction ?
        viewer.setAction(action)
        viewer.setPipeline(pipeline)
        viewer.play()
        
        return False


        
    def _createUi(self) :
        MainBox = self._createMainView()
                
        bbox = gtk.VBox()
        boxalign = gtk.Alignment(xalign=0.5, yalign=0.5)
        boxalign.add(bbox)
       

        self.ptv_viewer = PitiviViewer()
        bbox.pack_start(self.ptv_viewer, expand=False)
        
        bbox2 =  gtk.HBox()
        boxalign2 = gtk.Alignment(xalign=0.5, yalign=0.5)
        boxalign2.add(bbox2)

        self.capture_button = gtk.Button('start acquisition')
        self.capture_button.connect("clicked", self._start_acq_cb)
        self.capture_button.set_sensitive(True)
        bbox2.pack_start(self.capture_button, expand=False)
 
        self.capture_button = gtk.ToolButton(gtk.STOCK_ORIENTATION_LANDSCAPE)
        self.capture_button.connect("clicked", self._captureCb)
        self.capture_button.set_sensitive(True)
        bbox2.pack_start(self.capture_button, expand=False)
        
        self.capture_button = gtk.ToggleButton('with Scale ?')
        self.capture_button.connect("toggled", self._with_scale_Cb)
        self.capture_button.set_sensitive(True)
        bbox2.pack_start(self.capture_button, expand=False)

        bbox.pack_start(boxalign2, expand=False)
        
        
        self.viewer_frame.add(boxalign)
         
        self.webcam_props_widget = WebcamPropsWdg(self.wcam_props_frame)
       

        self.window.add(MainBox)
        self.window.show_all()

    def _createMainView(self) :
        self.window = gtk.Window()
        self.window.connect("delete-event", self._deleteCb)
        
        bbox = gtk.HBox()
        boxalign = gtk.Alignment(xalign=0.5, yalign=0.5)
        boxalign.add(bbox)
        
        self.viewer_frame =  gtk.Frame(label='Viewer')
        bbox.pack_start(self.viewer_frame, expand=False)
        
        self.wcam_props_frame =  gtk.Frame(label='webcam properties')
        bbox.pack_start(self.wcam_props_frame, expand=False)
        
        return boxalign


    def _caps_selected_cb(self, webcam_props, caps) :
        """ when a caps is selected restart acquisition """
        self.debug('Rx caps-selected : %s', caps.to_string() )
        self.wcam_params['wcam_filter'] = {
                'media_type' : caps,
                }
        
        self.prepare_viewer()

    def _webcam_changed_cb(self, wdg, webcam) :
        self.debug('Rx webcam-changed : %s', webcam )
        self.webcam_props.clear_all()
        self.wcam_params = {}
        
        
        _selected_device = None
        for _device in self._wcam_detect.webcam_devices :
            if _device['name'] == webcam :
                _selected_device = _device
                break
        if _selected_device != None :
            self.debug('Selected device %s ', _selected_device)
            self.wcam_params['wcam_driver'] = {
                    'element': _selected_device['v4ldriver'] ,
                    'properties' : {
                        'device':_selected_device['device-file'],
                        }
                    }
            
            self.prepare_viewer()
            self.webcam_props.prepare(self.factory.webcam_caps)

    def _deleteCb(self, unused_widget, unused_data):
        self.shutdown()

    def shutdown(self):
        #self.threads.stopAllThreads
        if self.ptv_viewer:
            self.ptv_viewer.destroy()
        self.mainloop.quit()
        return True

    def _start_acq_cb(self,data) :
        self.prepare_viewer()

    def _captureCb(self, data ):
        self.factory.capture()

    def _with_scale_Cb(self,widget) :
        self.with_scale = widget.get_active()
        self.prepare_viewer()
    

    

if __name__ == '__main__' :
    init_log()
    tester =Tester()
    tester.run()


